#include "ISModuleHandler.h"

using namespace zeitgeist;
using namespace std;

//-----------------------------------------------------------------------------
//	CISModuleHandler class
//-----------------------------------------------------------------------------

ISModuleHandler::ISModuleHandler()
{
	mISModules.clear();
}

ISModuleHandler::~ISModuleHandler()
{
	mISModules.clear();
}

int	ISModuleHandler::FindModuleIndex(BaseISModule *Module)
{
	//	vratime index do pola modulov, pre vstupny modul
	int i = 0;
	for (IS_MODULE_LIST::iterator iter = mISModules.begin(); iter != mISModules.end(); iter++, i++) {
		BaseISModule	*mod;

		mod = dynamic_cast<BaseISModule *>(*iter);
		if (mod == Module)
			return i;
	}

	return -1;
}

bool ISModuleHandler::IsModuleOrdered(int ModuleIndex)
{
	bool isOrdered = false;
	for (INT_LIST::iterator iter = mNotifyOrder.begin(); iter != mNotifyOrder.end(); iter++) {
		if ((int)(* iter) == ModuleIndex) {
			isOrdered = true;
			break;
		}
	}

	return isOrdered;
}

bool ISModuleHandler::SortModules()
{
        //      najdeme modul, ktory je zavysli, len na moduloch ktore uz su zoradene, opakujeme kym mame nejake nezaradene moduly
        //      ak mame nezaradene moduly, a nevieme najst ziadny ktory mozeme zaradit, vratime false

        mNotifyOrder.clear();

        while (mNotifyOrder.size() < mISModules.size()) {

                int orderedCount = 0;

                for (int i = 0; i < (int)mISModules.size(); i++) {

                        //      ak neni zaradeny, zistime ci ho mozeme zaradit
                        if (!IsModuleOrdered(i)) {
                                bool noDependency = true;
                                for (EDGE_LIST::iterator iter = mEdges.begin(); iter != mEdges.end(); iter++) {
                                        if ((dynamic_cast<Edge &>(* iter).DestIndex == i) && 
                                        	(!IsModuleOrdered(dynamic_cast<Edge &>(* iter).SourceIndex))) {
	                                        	noDependency = false;
                	                                break;
                                        }
                                }

                                //      ak neni zavisly
                                if (noDependency) {
                                        mNotifyOrder.push_back(i);
                                        orderedCount++;
                                }
                        }
                }

		//	ak sme nenasli ani jeden co mozeme zaradit, tak vyhlasime chybu
                if (orderedCount == 0)
                        return false;
	}
	
	return true;
}

bool ISModuleHandler::InsertModule(BaseISModule *Module)
{
	mISModules.push_back(Module);
	return true;
}

bool ISModuleHandler::InsertDependency(BaseISModule *Parent, BaseISModule *Child)
{
	Edge	edge;

	edge.SourceIndex = FindModuleIndex(Parent);
	edge.DestIndex = FindModuleIndex(Child);
		
	mEdges.push_back(edge);

	return true;
}

void ISModuleHandler::NotifyModules(int TimeStamp)
{
	//	pre vsetky polozky z pola zavolat DataChanged(TimeStamp)
	for (IS_MODULE_LIST::iterator iter = mISModules.begin(); iter != mISModules.end(); iter++) {
		dynamic_cast<BaseISModule *>(* iter)->DataChanged(TimeStamp);
	}
}

void ISModuleHandler::Vypis()
{
	printf("vazby:\n");
	for (EDGE_LIST::iterator iter = mEdges.begin(); iter != mEdges.end(); iter++) {
		printf("%i -> %i\n", dynamic_cast<Edge &>(* iter).SourceIndex, dynamic_cast<Edge &>(* iter).DestIndex);
	}

	printf("poradie:\n");
	for (INT_LIST::iterator iter = mNotifyOrder.begin(); iter != mNotifyOrder.end(); iter++) {
		printf("%i\n", (int)(* iter));
	}
}

