#include "Formations.h"

/*
 * Constructor/destructor
 */
Formations::Formations()
{
	mNow = 0;
	mNowRepeat = 0;
	mNumFormationStates = 0;
	mNumPlayers = 0;
	mPositions = NULL;
	mNumModes = NULL;
	mModes = NULL;
	mMaxPersist = NULL;
	mMultiplyer = NULL;
	mCheckBall = NULL;
	mCheckVapno = NULL;
	mBall = NULL;
	mBallMin = NULL;
	mBallMax = NULL;
	mRating = NULL;
	mPersist = NULL;
	mSumRating = 0;
}

Formations::~Formations()
{
	DeallocateStorage();
}

/*
 * Internal memory management.
 */
void Formations::AllocateStorage(unsigned numFormationStates, unsigned numPlayers)
{
	unsigned i, temp;

	temp = numFormationStates * numPlayers * 3;
	mPositions = (double *)malloc(temp * sizeof(double));
	for (i = 0; i < temp; i++)
		mPositions[i] = 0.0;

	mNumModes = (unsigned *)malloc(numFormationStates * sizeof(unsigned));
	for (i = 0; i < numFormationStates; i++)
		mNumModes[i] = 0;

	mModes = (TPlayMode **) malloc(numFormationStates * sizeof(TPlayMode *));
	for (i = 0; i < numFormationStates; i++)
		mModes[i] = NULL;

	mMaxPersist = (unsigned *)malloc(numFormationStates * sizeof(unsigned));
	for (i = 0; i < numFormationStates; i++)
		mMaxPersist[i] = 0;

	mMultiplyer = (double *)malloc(numFormationStates * sizeof(double));
	for (i = 0; i < numFormationStates; i++)
		mMultiplyer[i] = 0.0;

	mCheckBall = (unsigned *)malloc(numFormationStates * sizeof(unsigned));
	for (i = 0; i < numFormationStates; i++)
		mCheckBall[i] = 0;

	mCheckVapno = (unsigned *)malloc(numFormationStates * sizeof(unsigned));
	for (i = 0; i < numFormationStates; i++)
		mCheckVapno[i] = 0;

	mBall = (Vector3f *)malloc(numFormationStates * sizeof(Vector3f));
	for (i = 0; i < numFormationStates; i++)
		mBall[i] = *(new Vector3f(0.0f, 0.0f, 0.0f));

	mBallMin = (Vector3f *)malloc(numFormationStates * sizeof(Vector3f));
	for (i = 0; i < numFormationStates; i++)
		mBallMin[i] = *(new Vector3f(0.0f, 0.0f, 0.0f));

	mBallMax = (Vector3f *)malloc(numFormationStates * sizeof(Vector3f));
	for (i = 0; i < numFormationStates; i++)
		mBallMax[i] = *(new Vector3f(0.0f, 0.0f, 0.0f));

	mRating = (double *)malloc(numFormationStates * sizeof(double));
	for (i = 0; i < numFormationStates; i++)
		mRating[i] = 0.0;

	mPersist = (unsigned *)malloc(numFormationStates * sizeof(unsigned));
	for (i = 0; i < numFormationStates; i++)
		mPersist[i] = 0;
}

void Formations::DeallocateStorage()
{
	if (mPositions)
		free(mPositions);
	mPositions = NULL;

	if (mModes)
	{
		for (unsigned i = 0; i < mNumFormationStates; i++)
			if (mModes[i])
				free(mModes[i]);
		free(mModes);
		mModes = NULL;
	} // if (mModes)

	if (mNumModes)
		free(mNumModes);
	mNumModes = NULL;

	if (mMaxPersist)
		free(mMaxPersist);
	mMaxPersist = NULL;

	if (mMultiplyer)
		free(mMultiplyer);
	mMultiplyer = NULL;

	if (mCheckBall)
		free(mCheckBall);
	mCheckBall = NULL;

	if (mCheckVapno)
		free(mCheckVapno);
	mCheckVapno = NULL;

	if (mBall)
		free(mBall);
	mBall = NULL;

	if (mBallMin)
		free(mBallMin);
	mBallMin = NULL;

	if (mBallMax)
		free(mBallMax);
	mBallMax = NULL;

	if (mRating)
		free(mRating);
	mRating = NULL;

	if (mPersist)
		free(mPersist);
	mPersist = NULL;
	
	mNumFormationStates = 0;
	mNumPlayers = 0;
}

/*
 * Indexer.
 */
double &Formations::Positions(unsigned formationState, unsigned player, unsigned value)
{
	return mPositions[formationState * 3 * mNumPlayers + player * 3 + value];
}

/*
 * Overrided virtuals.
 */
void Formations::OnLink()
{
	// Ziskaj ukazovatel do stromu IS modulov.
	mISModuleHandler = shared_dynamic_cast<ISModuleHandler> (GetCore()->Get("/Agent/Sys/ISModuleHandler"));
	if (mISModuleHandler.get() == 0)
		GetLog()->Error() << "ERROR: (Formations) cannot find ISModuleHandler\n";
	
	// Registruj tento modul.
	mISModuleHandler->InsertModule(this);
	
	// Ziskaj ukazovatel na History modul. (Toto sa neskor prerobi na Predict modul).
	mHistory = shared_dynamic_cast<History> (GetCore()->Get("/Agent/Sys/History"));
	if (mHistory.get() == 0)
		GetLog()->Error() << "ERROR: (Formations) cannot find History model\n";
}

void Formations::OnUnLink()
{
	mISModuleHandler.reset();
	mHistory.reset();
}

/*
 * Initializations.
 */
void Formations::LoadDefaultFormations()
{
	DeallocateStorage();

	mNumFormationStates = 27;
	mNumPlayers = 11;
	AllocateStorage(mNumFormationStates, mNumPlayers);

	// Formation 1 ... before_kick_off 
	mMaxPersist[0] = 1;
	mMultiplyer[0] = 200;
	mNumModes[0] = 3;
	mModes[0] = (TPlayMode *)malloc((int)mNumModes[0] * sizeof(TPlayMode));
	mModes[0][0] = PM_BeforeKickOff;
	mModes[0][1] = PM_Goal_Right;
	mModes[0][2] = PM_Goal_Left;
	mCheckBall[0] = 0;
	mCheckVapno[0] = 0;

	Positions(0, 0, 0) = 0;// Valid.
	Positions(0, 1, 0) = 1;// Valid.
	Positions(0, 1, 1) = -0.5;// X.
	Positions(0, 1, 2) = 0;// Y.
	Positions(0, 2, 0) = 1;// Valid.
	Positions(0, 2, 1) = -0.5;// X.
	Positions(0, 2, 2) = 11.2;// Y.
	Positions(0, 3, 0) = 1;// Valid.
	Positions(0, 3, 1) = -0.5;// X.
	Positions(0, 3, 2) = -11.2;// Y.
	Positions(0, 4, 0) = 1;// Valid.
	Positions(0, 4, 1) = -6.8;// X.
	Positions(0, 4, 2) = 22.8;// Y.
	Positions(0, 5, 0) = 1;// Valid.
	Positions(0, 5, 1) = -9.2;// X.
	Positions(0, 5, 2) = 0;;// Y.
	Positions(0, 6, 0) = 1;// Valid.
	Positions(0, 6, 1) = -6.8;// X.
	Positions(0, 6, 2) = -22.8;// Y.
	Positions(0, 7, 0) = 1;// Valid.
	Positions(0, 7, 1) = -36;// X.
	Positions(0, 7, 2) = -20;// Y.
	Positions(0, 8, 0) = 1;// Valid.
	Positions(0, 8, 1) = -30;// X.
	Positions(0, 8, 2) = 8;// Y.
	Positions(0, 9, 0) = 1;// Valid.
	Positions(0, 9, 1) = -36;// X.
	Positions(0, 9, 2) = 20;// Y.
	Positions(0, 10, 0) = 1;// Valid.
	Positions(0, 10, 1) = -30;// X.
	Positions(0, 10, 2) = -8;// Y.

	// Formation 2 ... corner_kick_l_b
	mMaxPersist[1] = 20;
	mNumModes[1] = 1;
	mModes[1] = (TPlayMode *)malloc((int)mNumModes[1] * sizeof(TPlayMode));
	mModes[1][0] = PM_CORNER_KICK_LEFT;
	mCheckBall[1] = 1;
	mBall[1].x() = -52;
	mBall[1].y() = -34;
	mBallMin[1].x() = -5000;
	mBallMin[1].y() = -5000;
	mBallMax[1].x() = 5000;
	mBallMax[1].y() = 34;
	mMultiplyer[1] = 20;
	mCheckVapno[1] = 0;

	Positions(1, 0, 0) = 0; // Valid.
	Positions(1, 1, 0) = 1; // Valid.
	Positions(1, 1, 1) = -8; // X.
	Positions(1, 1, 2) = 0 ;// Y.
	Positions(1, 2, 0) = 1; // Valid.
	Positions(1, 2, 1) = -5.2; // X.
	Positions(1, 2, 2) = 22; // Y.
	Positions(1, 3, 0) = 1; // Valid.
	Positions(1, 3, 1) = -7; // X.
	Positions(1, 3, 2) = -20; // Y.
	Positions(1, 4, 0) = 1; // Valid.
	Positions(1, 4, 1) = -22; // X.
	Positions(1, 4, 2) = 14.4; // Y.
	Positions(1, 5, 0) = 1; // Valid.
	Positions(1, 5, 1) = -23.2; // X.
	Positions(1, 5, 2) = -6; // Y.
	Positions(1, 6, 0) = 1; // Valid.
	Positions(1, 6, 1) = -32; // X.
	Positions(1, 6, 2) = -23; // Y.
	Positions(1, 7, 0) = 1; // Valid.
	Positions(1, 7, 1) = -48; // X.
	Positions(1, 7, 2) = -22; // Y.
	Positions(1, 8, 0) = 1; // Valid.
	Positions(1, 8, 1) = -39.6; // X.
	Positions(1, 8, 2) = 1; // Y.
	Positions(1, 9, 0) = 1; // Valid.
	Positions(1, 9, 1) = -37; // X.
	Positions(1, 9, 2) = 13.6; // Y.
	Positions(1, 10, 0) = 1; // Valid.
	Positions(1, 10, 1) = -40.8; // X.
	Positions(1, 10, 2) = -12; // Y.

	// Formation 3 ... corner_kick_l_t
	mMaxPersist[2] = 20;
	mNumModes[2] = 1;
	mModes[2] = (TPlayMode *)malloc((int)mNumModes[2] * sizeof(TPlayMode));
	mModes[2][0] = PM_CORNER_KICK_LEFT;
	mCheckBall[2] = 1;
	mBall[2].x() = -52;
	mBall[2].y() = 34;
	mBallMin[2].x() = -5000;
	mBallMin[2].y() = -34;
	mBallMax[2].x() = 5000;
	mBallMax[2].y() = 5000;
	mMultiplyer[2] = 20;
	mCheckVapno[2] = 0;

	Positions(2, 0, 0) = 0; // Valid.
	Positions(2, 1, 0) = 1; // Valid.
	Positions(2, 1, 1) = -8; // X.
	Positions(2, 1, 2) = 0; // Y.
	Positions(2, 2, 0) = 1; // Valid.
	Positions(2, 2, 1) = -7; // X.
	Positions(2, 2, 2) = 20; // Y.
	Positions(2, 3, 0) = 1; // Valid.
	Positions(2, 3, 1) = -5.2; // X.
	Positions(2, 3, 2) = -22; // Y.
	Positions(2, 4, 0) = 1; // Valid.
	Positions(2, 4, 1) = -32; // X.
	Positions(2, 4, 2) = 23; // Y.
	Positions(2, 5, 0) = 1; // Valid.
	Positions(2, 5, 1) = -23.2; // X.
	Positions(2, 5, 2) = 6; // Y.
	Positions(2, 6, 0) = 1; // Valid.
	Positions(2, 6, 1) = -22; // X.
	Positions(2, 6, 2) = -14.4; // Y.
	Positions(2, 7, 0) = 1; // Valid.
	Positions(2, 7, 1) = -37; // X.
	Positions(2, 7, 2) = -13.6; // Y.
	Positions(2, 8, 0) = 1; // Valid.
	Positions(2, 8, 1) = -40.8; // X.
	Positions(2, 8, 2) = 12; // Y.
	Positions(2, 9, 0) = 1; // Valid.
	Positions(2, 9, 1) = -48; // X.
	Positions(2, 9, 2) = 22; // Y.
	Positions(2, 10, 0) = 1; // Valid.
	Positions(2, 10, 1) = -39.6; // X.
	Positions(2, 10, 2) = -1; // Y.

	// Formation 4 ... corner_kick_r_b 
	mMaxPersist[3] = 20;
	mNumModes[3] = 1;
	mModes[3] = (TPlayMode *)malloc((int)mNumModes[3] * sizeof(TPlayMode));
	mModes[3][0] = PM_CORNER_KICK_RIGHT;
	mCheckBall[3] = 1;
	mBall[3].x() = 52;
	mBall[3].y() = -34;
	mBallMin[3].x() = -5000;
	mBallMin[3].y() = -5000;
	mBallMax[3].x() = 5000;
	mBallMax[3].y() = 34;
	mMultiplyer[3] = 20;
	mCheckVapno[3] = 0;

	Positions(3, 0, 0) = 0; // Valid.
	Positions(3, 1, 0) = 1; // Valid.
	Positions(3, 1, 1) = 36; // X.
	Positions(3, 1, 2) = -6.8; // Y.
	Positions(3, 2, 0) = 1; // Valid.
	Positions(3, 2, 1) = 36; // X.
	Positions(3, 2, 2) = 8.8; // Y.
	Positions(3, 3, 0) = 1; // Valid.
	Positions(3, 3, 1) = 45.2; // X.
	Positions(3, 3, 2) = -24.8; // Y.
	Positions(3, 4, 0) = 1; // Valid.
	Positions(3, 4, 1) = 18.4; // X.
	Positions(3, 4, 2) = 18.4; // Y.
	Positions(3, 5, 0) = 1; // Valid.
	Positions(3, 5, 1) = 18.4; // X.
	Positions(3, 5, 2) = -5; // Y.
	Positions(3, 6, 0) = 1; // Valid.
	Positions(3, 6, 1) = 24; // X.
	Positions(3, 6, 2) = -21.2; // Y.
	Positions(3, 7, 0) = 0; // Valid.
	Positions(3, 8, 0) = 0; // Valid.
	Positions(3, 9, 0) = 0; // Valid.
	Positions(3, 10, 0) = 0; // Valid.

	// Formation 5 ... corner_kick_r_t 
	mMaxPersist[4] = 20;
	mNumModes[4] = 1;
	mModes[4] = (TPlayMode *)malloc((int)mNumModes[4] * sizeof(TPlayMode));
	mModes[4][0] = PM_CORNER_KICK_RIGHT;
	mCheckBall[4] = 1;
	mBall[4].x() = 52;
	mBall[4].y() = 34;
	mBallMin[4].x() = -5000;
	mBallMin[4].y() = -34;
	mBallMax[4].x() = 5000;
	mBallMax[4].y() = 5000;
	mMultiplyer[4] = 20;
	mCheckVapno[4] = 0;

	Positions(4, 0, 0) = 0; // Valid.
	Positions(4, 1, 0) = 1; // Valid.
	Positions(4, 1, 1) = 36; // X.
	Positions(4, 1, 2) = 6.8; // Y.
	Positions(4, 2, 0) = 1; // Valid.
	Positions(4, 2, 1) = 45.2; // X.
	Positions(4, 2, 2) = 24.8; // Y.
	Positions(4, 3, 0) = 1; // Valid.
	Positions(4, 3, 1) = 36; // X.
	Positions(4, 3, 2) = -8.8; // Y.
	Positions(4, 4, 0) = 1; // Valid.
	Positions(4, 4, 1) = 24; // X.
	Positions(4, 4, 2) = 21.2; // Y.
	Positions(4, 5, 0) = 1; // Valid.
	Positions(4, 5, 1) = 18.4; // X.
	Positions(4, 5, 2) = 5; // Y.
	Positions(4, 6, 0) = 1; // Valid.
	Positions(4, 6, 1) = 18.4; // X.
	Positions(4, 6, 2) = -18.4; // Y.
	Positions(4, 7, 0) = 0; // Valid.
	Positions(4, 8, 0) = 0; // Valid.
	Positions(4, 9, 0) = 0; // Valid.
	Positions(4, 10, 0) = 0; // Valid.

	// Formation 6 ... free_kick_l_b
	mMaxPersist[5] = 10;
	mNumModes[5] = 1;
	mModes[5] = (TPlayMode *)malloc((int)mNumModes[5] * sizeof(TPlayMode));
	mModes[5][0] = PM_FREE_KICK_LEFT;
	mCheckBall[5] = 1;
	mBall[5].x() = -52;
	mBall[5].y() = -34;
	mBallMin[5].x() = -5000;
	mBallMin[5].y() = -5000;
	mBallMax[5].x() = -18;
	mBallMax[5].y() = -7;
	mMultiplyer[5] = 10;
	mCheckVapno[5] = 0;

	Positions(5, 0, 0) = 0; // Valid.
	Positions(5, 1, 0) = 0; // Valid.
	Positions(5, 2, 0) = 0; // Valid.
	Positions(5, 3, 0) = 0; // Valid.
	Positions(5, 4, 0) = 0; // Valid.
	Positions(5, 5, 0) = 0; // Valid.
	Positions(5, 6, 0) = 1; // Valid.
	Positions(5, 6, 1) = -28; // X.
	Positions(5, 6, 2) = -21; // Y.
	Positions(5, 7, 0) = 1; // Valid.
	Positions(5, 7, 1) = -52; // X.
	Positions(5, 7, 2) = -16.8; // Y.
	Positions(5, 8, 0) = 1; // Valid.
	Positions(5, 8, 1) = -38.4; // X.
	Positions(5, 8, 2) = -1; // Y.
	Positions(5, 9, 0) = 1; // Valid.
	Positions(5, 9, 1) = -37.2; // X.
	Positions(5, 9, 2) = 14; // Y.
	Positions(5, 10, 0) = 1; // Valid.
	Positions(5, 10, 1) = -40; // X.
	Positions(5, 10, 2) = -13.2; // Y.

	// Formation 7 ... free_kick_l_b 
	mMaxPersist[6] = 10;
	mNumModes[6] = 1;
	mModes[6] = (TPlayMode *)malloc((int)mNumModes[6] * sizeof(TPlayMode));
	mModes[6][0] = PM_FREE_KICK_LEFT;
	mCheckBall[6] = 1;
	mBall[6].x() = -36;
	mBall[6].y() = -20.16;
	mBallMin[6].x() = -5000;
	mBallMin[6].y() = -5000;
	mBallMax[6].x() = 0;
	mBallMax[6].y() = 20.16;
	mMultiplyer[6] = 10;
	mCheckVapno[6] = 0;

	Positions(6, 0, 0) = 0; // Valid.
	Positions(6, 1, 0) = 1; // Valid.
	Positions(6, 1, 1) = -2; // X.
	Positions(6, 1, 2) = 0; // Y.
	Positions(6, 2, 0) = 1; // Valid.
	Positions(6, 2, 1) = -8.8; // X.
	Positions(6, 2, 2) = 22; // Y.
	Positions(6, 3, 0) = 1; // Valid.
	Positions(6, 3, 1) = -9; // X.
	Positions(6, 3, 2) = -19; // Y.
	Positions(6, 4, 0) = 1; // Valid.
	Positions(6, 4, 1) = -20; // X.
	Positions(6, 4, 2) = 20; // Y.
	Positions(6, 5, 0) = 1; // Valid.
	Positions(6, 5, 1) = -25.6; // X.
	Positions(6, 5, 2) = 0; // Y.
	Positions(6, 6, 0) = 1; // Valid.
	Positions(6, 6, 1) = -21; // X.
	Positions(6, 6, 2) = -20; // Y.
	Positions(6, 7, 0) = 1; // Valid.
	Positions(6, 7, 1) = -40.8; // X.
	Positions(6, 7, 2) = -13.6; // Y.
	Positions(6, 8, 0) = 1; // Valid.
	Positions(6, 8, 1) = -40; // X.
	Positions(6, 8, 2) = -2; // Y.
	Positions(6, 9, 0) = 1; // Valid.
	Positions(6, 9, 1) = -36; // X.
	Positions(6, 9, 2) = 10; // Y.
	Positions(6, 10, 0) = 1; // Valid.
	Positions(6, 10, 1) = -32.4; // X.
	Positions(6, 10, 2) = -8.8; // Y.

	// Formation 8 ... free_kick_l_t
	mMaxPersist[7] = 10;
	mNumModes[7] = 1;
	mModes[7] = (TPlayMode *)malloc((int)mNumModes[7] * sizeof(TPlayMode));
	mModes[7][0] = PM_FREE_KICK_LEFT;
	mCheckBall[7] = 1;
	mBall[7].x() = -52;
	mBall[7].y() = 34;
	mBallMin[7].x() = -5000;
	mBallMin[7].y() = 7;
	mBallMax[7].x() = -18;
	mBallMax[7].y() = 5000;
	mMultiplyer[7] = 10;
	mCheckVapno[7] = 0;

	Positions(7, 0, 0) = 0; // Valid.
	Positions(7, 1, 0) = 0; // Valid.
	Positions(7, 2, 0) = 0; // Valid.
	Positions(7, 3, 0) = 0; // Valid.
	Positions(7, 4, 0) = 1; // Valid.
	Positions(7, 4, 1) = -28; // X.
	Positions(7, 4, 2) = 21; // Y.
	Positions(7, 5, 0) = 0; // Valid.
	Positions(7, 6, 0) = 0; // Valid.
	Positions(7, 7, 0) = 1; // Valid.
	Positions(7, 7, 1) = -37.2; // X.
	Positions(7, 7, 2) = -14; // Y.
	Positions(7, 8, 0) = 1; // Valid.
	Positions(7, 8, 1) = -40; // X.
	Positions(7, 8, 2) = 13.2; // Y.
	Positions(7, 9, 0) = 1; // Valid.
	Positions(7, 9, 1) = -52; // X.
	Positions(7, 9, 2) = 16.8; // Y.
	Positions(7, 10, 0) = 1; // Valid.
	Positions(7, 10, 1) = -38.4; // X.
	Positions(7, 10, 2) = 1; // Y.

	// Formation 9 ... free_kick_l_t 
	mMaxPersist[8] = 10;
	mNumModes[8] = 1;
	mModes[8] = (TPlayMode *)malloc((int)mNumModes[8] * sizeof(TPlayMode));
	mModes[8][0] = PM_FREE_KICK_LEFT;
	mCheckBall[8] = 1;
	mBall[8].x() = -36;
	mBall[8].y() = 20.16;
	mBallMin[8].x() = -5000;
	mBallMin[8].y() = -20.16;
	mBallMax[8].x() = 0;
	mBallMax[8].y() = 5000;
	mMultiplyer[8] = 10;
	mCheckVapno[8] = 0;

	Positions(8, 0, 0) = 0; // Valid.
	Positions(8, 1, 0) = 1; // Valid.
	Positions(8, 1, 1) = -2; // X.
	Positions(8, 1, 2) = 0; // Y.
	Positions(8, 3, 0) = 1; // Valid.
	Positions(8, 3, 1) = -8.8; // X.
	Positions(8, 3, 2) = -22; // Y.
	Positions(8, 2, 0) = 1; // Valid.
	Positions(8, 2, 1) = -9; // X.
	Positions(8, 2, 2) = 19; // Y.
	Positions(8, 6, 0) = 1; // Valid.
	Positions(8, 6, 1) = -20; // X.
	Positions(8, 6, 2) = -20; // Y.
	Positions(8, 5, 0) = 1; // Valid.
	Positions(8, 5, 1) = -25.6; // X.
	Positions(8, 5, 2) = 0; // Y.
	Positions(8, 4, 0) = 1; // Valid.
	Positions(8, 4, 1) = -21; // X.
	Positions(8, 4, 2) = 20; // Y.
	Positions(8, 9, 0) = 1; // Valid.
	Positions(8, 9, 1) = -40.8; // X.
	Positions(8, 9, 2) = 13.6; // Y.
	Positions(8, 10, 0) = 1; // Valid.
	Positions(8, 10, 1) = -40; // X.
	Positions(8, 10, 2) = 2; // Y.
	Positions(8, 7, 0) = 1; // Valid.
	Positions(8, 7, 1) = -36; // X.
	Positions(8, 7, 2) = -10; // Y.
	Positions(8, 8, 0) = 1; // Valid.
	Positions(8, 8, 1) = -32.4; // X.
	Positions(8, 8, 2) = 8.8; // Y.

	// Formation 10 ... goal_kick_l_b 
	mMaxPersist[9] = 20;
	mNumModes[9] = 1;
	mModes[9] = (TPlayMode *)malloc((int)mNumModes[9] * sizeof(TPlayMode));
	mModes[9][0] = PM_GOAL_KICK_LEFT;
	mCheckBall[9] = 1;
	mBall[9].x() = 47;
	mBall[9].y() = -7;
	mBallMin[9].x() = -5000;
	mBallMin[9].y() = -5000;
	mBallMax[9].x() = 5000;
	mBallMax[9].y() = 7;
	mMultiplyer[9] = 10;
	mCheckVapno[9] = 0;

	Positions(9, 0, 0) = 0; // Valid.
	Positions(9, 1, 0) = 1; // Valid.
	Positions(9, 1, 1) = 24; // X.
	Positions(9, 1, 2) = 0; // Y.
	Positions(9, 3, 0) = 1; // Valid.
	Positions(9, 3, 1) = 18; // X.
	Positions(9, 3, 2) = -19.2; // Y.
	Positions(9, 2, 0) = 1; // Valid.
	Positions(9, 2, 1) = 18; // X.
	Positions(9, 2, 2) = 19.2; // Y.
	Positions(9, 6, 0) = 1; // Valid.
	Positions(9, 6, 1) = -9.2; // X.
	Positions(9, 6, 2) = -18.4; // Y.
	Positions(9, 5, 0) = 1; // Valid.
	Positions(9, 5, 1) = -12; // X.
	Positions(9, 5, 2) = 0; // Y.
	Positions(9, 4, 0) = 1; // Valid.
	Positions(9, 4, 1) = -9.2; // X.
	Positions(9, 4, 2) = 18.4; // Y.
	Positions(9, 9, 0) = 1; // Valid.
	Positions(9, 9, 1) = -36; // X.
	Positions(9, 9, 2) = 20; // Y.
	Positions(9, 10, 0) = 1; // Valid.
	Positions(9, 10, 1) = -30; // X.
	Positions(9, 10, 2) = -8; // Y.
	Positions(9, 7, 0) = 1; // Valid.
	Positions(9, 7, 1) = -36; // X.
	Positions(9, 7, 2) = -20; // Y.
	Positions(9, 8, 0) = 1; // Valid.
	Positions(9, 8, 1) = -30; // X.
	Positions(9, 8, 2) = 8; // Y.

	// Formation 11 ... goal_kick_r_t 
	mMaxPersist[10] = 20;
	mNumModes[10] = 1;
	mModes[10] = (TPlayMode *)malloc((int)mNumModes[10] * sizeof(TPlayMode));
	mModes[10][0] = PM_GOAL_KICK_RIGHT;
	mCheckBall[10] = 1;
	mBall[10].x() = 47;
	mBall[10].y() = 7;
	mBallMin[10].x() = -5000;
	mBallMin[10].y() = -7;
	mBallMax[10].x() = 5000;
	mBallMax[10].y() = 5000;
	mMultiplyer[10] = 10;
	mCheckVapno[10] = 0;

	Positions(10, 0, 0) = 0; // Valid.
	Positions(10, 1, 0) = 1; // Valid.
	Positions(10, 1, 1) = 24; // X.
	Positions(10, 1, 2) = 0; // Y.
	Positions(10, 2, 0) = 1; // Valid.
	Positions(10, 2, 1) = 18; // X.
	Positions(10, 2, 2) = 19.2; // Y.
	Positions(10, 3, 0) = 1; // Valid.
	Positions(10, 3, 1) = 18; // X.
	Positions(10, 3, 2) = -19.2; // Y.
	Positions(10, 4, 0) = 1; // Valid.
	Positions(10, 4, 1) = -9.2; // X.
	Positions(10, 4, 2) = 18.4; // Y.
	Positions(10, 5, 0) = 1; // Valid.
	Positions(10, 5, 1) = -12; // X.
	Positions(10, 5, 2) = 0; // Y.
	Positions(10, 6, 0) = 1; // Valid.
	Positions(10, 6, 1) = -9.2; // X.
	Positions(10, 6, 2) = -18.4; // Y.
	Positions(10, 7, 0) = 1; // Valid.
	Positions(10, 7, 1) = -36; // X.
	Positions(10, 7, 2) = -20; // Y.
	Positions(10, 8, 0) = 1; // Valid.
	Positions(10, 8, 1) = -30; // X.
	Positions(10, 8, 2) = 8; // Y.
	Positions(10, 9, 0) = 1; // Valid.
	Positions(10, 9, 1) = -36; // X.
	Positions(10, 9, 2) = 20; // Y.
	Positions(10, 10, 0) = 1; // Valid.
	Positions(10, 10, 1) = -30; // X.
	Positions(10, 10, 2) = -8; // Y.

	// Formation 12 ... goal_kick_l_b 
	mMaxPersist[11] = 20;
	mNumModes[11] = 1;
	mModes[11] = (TPlayMode *)malloc((int)mNumModes[11] * sizeof(TPlayMode));
	mModes[11][0] = PM_GOAL_KICK_LEFT;
	mCheckBall[11] = 1;
	mBall[11].x() = -47;
	mBall[11].y() = -7;
	mBallMin[11].x() = -5000;
	mBallMin[11].y() = -5000;
	mBallMax[11].x() = 5000;
	mBallMax[11].y() = 7;
	mMultiplyer[11] = 10;
	mCheckVapno[11] = 0;

	Positions(11, 0, 0) = 0; // Valid.
	Positions(11, 1, 0) = 1; // Valid.
	Positions(11, 1, 1) = 0; // X.
	Positions(11, 1, 2) = 0; // Y.
	Positions(11, 2, 0) = 1; // Valid.
	Positions(11, 2, 1) = 0; // X.
	Positions(11, 2, 2) = 22.4; // Y.
	Positions(11, 3, 0) = 1; // Valid.
	Positions(11, 3, 1) = 0; // X.
	Positions(11, 3, 2) = -25.6; // Y.
	Positions(11, 4, 0) = 1; // Valid.
	Positions(11, 4, 1) = -19.2; // X.
	Positions(11, 4, 2) = 16; // Y.
	Positions(11, 5, 0) = 1; // Valid.
	Positions(11, 5, 1) = -19.2; // X.
	Positions(11, 5, 2) = -2; // Y.
	Positions(11, 6, 0) = 1; // Valid.
	Positions(11, 6, 1) = -17.2; // X.
	Positions(11, 6, 2) = -20; // Y.
	Positions(11, 7, 0) = 1; // Valid.
	Positions(11, 7, 1) = -36; // X.
	Positions(11, 7, 2) = -20; // Y.
	Positions(11, 8, 0) = 1; // Valid.
	Positions(11, 8, 1) = -36; // X.
	Positions(11, 8, 2) = 8; // Y.
	Positions(11, 9, 0) = 1; // Valid.
	Positions(11, 9, 1) = -36; // X.
	Positions(11, 9, 2) = 20; // Y.
	Positions(11, 10, 0) = 1; // Valid.
	Positions(11, 10, 1) = -30; // X.
	Positions(11, 10, 2) = -8; // Y.

	// Formation 13 ... goal_kick_l_t 
	mMaxPersist[12] = 20;
	mNumModes[12] = 1;
	mModes[12] = (TPlayMode *)malloc((int)mNumModes[12] * sizeof(TPlayMode));
	mModes[12][0] = PM_GOAL_KICK_LEFT;
	mCheckBall[12] = 1;
	mBall[12].x() = -47;
	mBall[12].y() = 7;
	mBallMin[12].x() = -5000;
	mBallMin[12].y() = -7;
	mBallMax[12].x() = 5000;
	mBallMax[12].y() = 5000;
	mMultiplyer[12] = 10;
	mCheckVapno[12] = 0;

	Positions(12, 0, 0) = 0; // Valid.
	Positions(12, 1, 0) = 1; // Valid.
	Positions(12, 1, 1) = 0; // X.
	Positions(12, 1, 2) = 0; // Y.
	Positions(12, 2, 0) = 1; // Valid.
	Positions(12, 2, 1) = 0; // X.
	Positions(12, 2, 2) = 25.6; // Y.
	Positions(12, 3, 0) = 1; // Valid.
	Positions(12, 3, 1) = 0; // X.
	Positions(12, 3, 2) = -22.4; // Y.
	Positions(12, 4, 0) = 1; // Valid.
	Positions(12, 4, 1) = -17.2; // X.
	Positions(12, 4, 2) = 20; // Y.
	Positions(12, 5, 0) = 1; // Valid.
	Positions(12, 5, 1) = -19.2; // X.
	Positions(12, 5, 2) = 2; // Y.
	Positions(12, 6, 0) = 1; // Valid.
	Positions(12, 6, 1) = -19.2; // X.
	Positions(12, 6, 2) = -16; // Y.
	Positions(12, 7, 0) = 1; // Valid.
	Positions(12, 7, 1) = -36; // X.
	Positions(12, 7, 2) = -20; // Y.
	Positions(12, 8, 0) = 1; // Valid.
	Positions(12, 8, 1) = -30; // X.
	Positions(12, 8, 2) = 8; // Y.
	Positions(12, 9, 0) = 1; // Valid.
	Positions(12, 9, 1) = -36; // X.
	Positions(12, 9, 2) = 20; // Y.
	Positions(12, 10, 0) = 1; // Valid.
	Positions(12, 10, 1) = -36; // X.
	Positions(12, 10, 2) = -8; // Y.

	// Formation 14 ... kick_off_left 
	mMaxPersist[13] = 15;
	mNumModes[13] = 1;
	mModes[13] = (TPlayMode *)malloc((int)mNumModes[13] * sizeof(TPlayMode));
	mModes[13][0] = PM_KickOff_Left;
	mCheckBall[13] = 0;
	mBall[13].x() = 0;
	mBall[13].y() = 0;
	mBallMin[13].x() = 0;
	mBallMin[13].y() = 0;
	mBallMax[13].x() = 0;
	mBallMax[13].y() = 0;
	mMultiplyer[13] = 200;
	mCheckVapno[13] = 0;

	Positions(13, 0, 0) = 0; // Valid.
	Positions(13, 1, 0) = 1; // Valid.
	Positions(13, 1, 1) = -7.6; // X.
	Positions(13, 1, 2) = 5.6; // Y.
	Positions(13, 2, 0) = 1; // Valid.
	Positions(13, 2, 1) = -0.5; // X.
	Positions(13, 2, 2) = 21.6; // Y.
	Positions(13, 3, 0) = 1; // Valid.
	Positions(13, 3, 1) = -0.5; // X.
	Positions(13, 3, 2) = -21.6; // Y.
	Positions(13, 4, 0) = 1; // Valid.
	Positions(13, 4, 1) = -14; // X.
	Positions(13, 4, 2) = 20.8; // Y.
	Positions(13, 5, 0) = 1; // Valid.
	Positions(13, 5, 1) = -7.6; // X.
	Positions(13, 5, 2) = -5.6; // Y.
	Positions(13, 6, 0) = 1; // Valid.
	Positions(13, 6, 1) = -14; // X.
	Positions(13, 6, 2) = -20.8; // Y.
	Positions(13, 7, 0) = 1; // Valid.
	Positions(13, 7, 1) = -36; // X.
	Positions(13, 7, 2) = -20; // Y.
	Positions(13, 8, 0) = 1; // Valid.
	Positions(13, 8, 1) = -30; // X.
	Positions(13, 8, 2) = 8; // Y.
	Positions(13, 9, 0) = 1; // Valid.
	Positions(13, 9, 1) = -36; // X.
	Positions(13, 9, 2) = 20; // Y.
	Positions(13, 10, 0) = 1; // Valid.
	Positions(13, 10, 1) = -30; // X.
	Positions(13, 10, 2) = -8; // Y.

	// Formation 15 ... kick_off_right 
	mMaxPersist[14] = 5;
	mNumModes[14] = 1;
	mModes[14] = (TPlayMode *)malloc((int)mNumModes[14] * sizeof(TPlayMode));
	mModes[14][0] = PM_KickOff_Right;
	mCheckBall[14] = 0;
	mBall[14].x() = 0;
	mBall[14].y() = 0;
	mBallMin[14].x() = 0;
	mBallMin[14].y() = 0;
	mBallMax[14].x() = 0;
	mBallMax[14].y() = 0;
	mMultiplyer[14] = 200;
	mCheckVapno[14] = 0;

	Positions(14, 0, 0) = 0; // Valid.
	Positions(14, 1, 0) = 1; // Valid.
	Positions(14, 1, 1) = -0.5; // X.
	Positions(14, 1, 2) = 0; // Y.
	Positions(14, 2, 0) = 1; // Valid.
	Positions(14, 2, 1) = -0.5; // X.
	Positions(14, 2, 2) = 11.2; // Y.
	Positions(14, 3, 0) = 1; // Valid.
	Positions(14, 3, 1) = -0.5; // X.
	Positions(14, 3, 2) = -11.2; // Y.
	Positions(14, 4, 0) = 1; // Valid.
	Positions(14, 4, 1) = -6.8; // X.
	Positions(14, 4, 2) = 22.8; // Y.
	Positions(14, 5, 0) = 1; // Valid.
	Positions(14, 5, 1) = -9.2; // X.
	Positions(14, 5, 2) = 0; // Y.
	Positions(14, 6, 0) = 1; // Valid.
	Positions(14, 6, 1) = -6.8; // X.
	Positions(14, 6, 2) = -22.8; // Y.
	Positions(14, 7, 0) = 1; // Valid.
	Positions(14, 7, 1) = -36; // X.
	Positions(14, 7, 2) = -20; // Y.
	Positions(14, 8, 0) = 1; // Valid.
	Positions(14, 8, 1) = -30; // X.
	Positions(14, 8, 2) = 8; // Y.
	Positions(14, 9, 0) = 1; // Valid.
	Positions(14, 9, 1) = -36; // X.
	Positions(14, 9, 2) = 20; // Y.
	Positions(14, 10, 0) = 1; // Valid.
	Positions(14, 10, 1) = -30; // X.
	Positions(14, 10, 2) = -8; // Y.

	// Formation 16 ... play_on_l_b_3 
	mMaxPersist[15] = 1;
	mNumModes[15] = 0;
	mModes[15] = (TPlayMode *)malloc((int)mNumModes[15] * sizeof(TPlayMode));
	mCheckBall[15] = 1;
	mBall[15].x() = -52;
	mBall[15].y() = -34;
	mBallMin[15].x() = -5000;
	mBallMin[15].y() = -5000;
	mBallMax[15].x() = 0;
	mBallMax[15].y() = 34;
	mMultiplyer[15] = 3;
	mCheckVapno[15] = 1;

	Positions(15, 0, 0) = 0; // Valid.
	Positions(15, 1, 0) = 0; // Valid.
	Positions(15, 2, 0) = 0; // Valid.
	Positions(15, 3, 0) = 0; // Valid.
	Positions(15, 4, 0) = 0; // Valid.
	Positions(15, 5, 0) = 0; // Valid.
	Positions(15, 6, 0) = 0; // Valid.
	Positions(15, 7, 0) = 1; // Valid.
	Positions(15, 7, 1) = -48; // X.
	Positions(15, 7, 2) = -20; // Y.
	Positions(15, 8, 0) = 1; // Valid.
	Positions(15, 8, 1) = -41.5; // X.
	Positions(15, 8, 2) = 2; // Y.
	Positions(15, 9, 0) = 1; // Valid.
	Positions(15, 9, 1) = -41.5; // X.
	Positions(15, 9, 2) = 12; // Y.
	Positions(15, 10, 0) = 1; // Valid.
	Positions(15, 10, 1) = -41.5; // X.
	Positions(15, 10, 2) = -10; // Y.

	// Formation 17 ... play_on_l_b 
	mMaxPersist[16] = 1;
	mNumModes[16] = 0;
	mModes[16] = (TPlayMode *)malloc((int)mNumModes[16] * sizeof(TPlayMode));
	mCheckBall[16] = 1;
	mBall[16].x() = -52;
	mBall[16].y() = -34;
	mBallMin[16].x() = -5000;
	mBallMin[16].y() = -5000;
	mBallMax[16].x() = 0;
	mBallMax[16].y() = 34;
	mMultiplyer[16] = 1;
	mCheckVapno[16] = 0;

	Positions(16, 0, 0) = 0; // Valid.
	Positions(16, 1, 0) = 1; // Valid.
	Positions(16, 1, 1) = 0; // X.
	Positions(16, 1, 2) = 0; // Y.
	Positions(16, 2, 0) = 1; // Valid.
	Positions(16, 2, 1) = 0; // X.
	Positions(16, 2, 2) = 22; // Y.
	Positions(16, 3, 0) = 1; // Valid.
	Positions(16, 3, 1) = -4; // X.
	Positions(16, 3, 2) = -22.8; // Y.
	Positions(16, 4, 0) = 1; // Valid.
	Positions(16, 4, 1) = -20; // X.
	Positions(16, 4, 2) = 19.2; // Y.
	Positions(16, 5, 0) = 1; // Valid.
	Positions(16, 5, 1) = -21.2; // X.
	Positions(16, 5, 2) = -2; // Y.
	Positions(16, 6, 0) = 1; // Valid.
	Positions(16, 6, 1) = -30; // X.
	Positions(16, 6, 2) = -24; // Y.
	Positions(16, 7, 0) = 1; // Valid.
	Positions(16, 7, 1) = -50; // X.
	Positions(16, 7, 2) = -34; // Y.
	Positions(16, 8, 0) = 1; // Valid.
	Positions(16, 8, 1) = -45; // X.
	Positions(16, 8, 2) = -13; // Y.
	Positions(16, 9, 0) = 1; // Valid.
	Positions(16, 9, 1) = -45; // X.
	Positions(16, 9, 2) = 0.2; // Y.
	Positions(16, 10, 0) = 1; // Valid.
	Positions(16, 10, 1) = -50; // X.
	Positions(16, 10, 2) = -25.4; // Y.

	// Formation 18 ... play_on_l_t_3 
	mMaxPersist[17] = 1;
	mNumModes[17] = 0;
	mModes[17] = (TPlayMode *)malloc((int)mNumModes[17] * sizeof(TPlayMode));
	mCheckBall[17] = 1;
	mBall[17].x() = -52;
	mBall[17].y() = 34;
	mBallMin[17].x() = -5000;
	mBallMin[17].y() = -34;
	mBallMax[17].x() = 0;
	mBallMax[17].y() = 5000;
	mMultiplyer[17] = 3;
	mCheckVapno[17] = 1;
	
	Positions(17, 0, 0) = 0; // Valid.
	Positions(17, 1, 0) = 0; // Valid.
	Positions(17, 2, 0) = 0; // Valid.
	Positions(17, 3, 0) = 0; // Valid.
	Positions(17, 4, 0) = 0; // Valid.
	Positions(17, 5, 0) = 0; // Valid.
	Positions(17, 6, 0) = 0; // Valid.
	Positions(17, 7, 0) = 1; // Valid.
	Positions(17, 7, 1) = -41.5; // X.
	Positions(17, 7, 2) = -12; // Y.
	Positions(17, 8, 0) = 1; // Valid.
	Positions(17, 8, 1) = -41.5; // X.
	Positions(17, 8, 2) = 10; // Y.
	Positions(17, 9, 0) = 1; // Valid.
	Positions(17, 9, 1) = -48; // X.
	Positions(17, 9, 2) = 20; // Y.
	Positions(17, 10, 0) = 1; // Valid.
	Positions(17, 10, 1) = -41.5; // X.
	Positions(17, 10, 2) = -2; // Y.

	// Formation 19 ... play_on_l_t 
	mMaxPersist[18] = 1;
	mNumModes[18] = 0;
	mModes[18] = (TPlayMode *)malloc((int)mNumModes[18] * sizeof(TPlayMode));
	mCheckBall[18] = 1;
	mBall[18].x() = -52;
	mBall[18].y() = 34;
	mBallMin[18].x() = -5000;
	mBallMin[18].y() = -34;
	mBallMax[18].x() = 0;
	mBallMax[18].y() = 5000;
	mMultiplyer[18] = 1;
	mCheckVapno[18] = 0;

	Positions(18, 0, 0) = 0; // Valid.
	Positions(18, 1, 0) = 1; // Valid.
	Positions(18, 1, 1) = 0; // X.
	Positions(18, 1, 2) = 0; // Y.
	Positions(18, 2, 0) = 1; // Valid.
	Positions(18, 2, 1) = -4; // X.
	Positions(18, 2, 2) = 22.8; // Y.
	Positions(18, 3, 0) = 1; // Valid.
	Positions(18, 3, 1) = 0; // X.
	Positions(18, 3, 2) = -22; // Y.
	Positions(18, 4, 0) = 1; // Valid.
	Positions(18, 4, 1) = -30; // X.
	Positions(18, 4, 2) = 24; // Y.
	Positions(18, 5, 0) = 1; // Valid.
	Positions(18, 5, 1) = -21.2; // X.
	Positions(18, 5, 2) = 2; // Y.
	Positions(18, 6, 0) = 1; // Valid.
	Positions(18, 6, 1) = -20; // X.
	Positions(18, 6, 2) = -19.2; // Y.
	Positions(18, 7, 0) = 1; // Valid.
	Positions(18, 7, 1) = -45; // X.
	Positions(18, 7, 2) = -0.2; // Y.
	Positions(18, 8, 0) = 1; // Valid.
	Positions(18, 8, 1) = -50; // X.
	Positions(18, 8, 2) = 25.4; // Y.
	Positions(18, 9, 0) = 1; // Valid.
	Positions(18, 9, 1) = -50; // X.
	Positions(18, 9, 2) = 34; // Y.
	Positions(18, 10, 0) = 1; // Valid.
	Positions(18, 10, 1) = -45; // X.
	Positions(18, 10, 2) = 13; // Y.

	// Formation 20 ... play_on_m_b_3 
	mMaxPersist[19] = 1;
	mNumModes[19] = 0;
	mModes[19] = (TPlayMode *)malloc((int)mNumModes[19] * sizeof(TPlayMode));
	mCheckBall[19] = 1;
	mBall[19].x() = 0;
	mBall[19].y() = -34;
	mBallMin[19].x() = -52;
	mBallMin[19].y() = -5000;
	mBallMax[19].x() = 52;
	mBallMax[19].y() = 34;
	mMultiplyer[19] = 3;
	mCheckVapno[19] = 1;

	Positions(19, 0, 0) = 0; // Valid.
	Positions(19, 1, 0) = 0; // Valid.
	Positions(19, 2, 0) = 0; // Valid.
	Positions(19, 3, 0) = 0; // Valid.
	Positions(19, 4, 0) = 0; // Valid.
	Positions(19, 5, 0) = 0; // Valid.
	Positions(19, 6, 0) = 0; // Valid.
	Positions(19, 7, 0) = 1; // Valid.
	Positions(19, 7, 1) = -28.8; // X.
	Positions(19, 7, 2) = -23.2; // Y.
	Positions(19, 8, 0) = 1; // Valid.
	Positions(19, 8, 1) = -31.2; // X.
	Positions(19, 8, 2) = 0; // Y.
	Positions(19, 9, 0) = 1; // Valid.
	Positions(19, 9, 1) = -33; // X.
	Positions(19, 9, 2) = 14; // Y.
	Positions(19, 10, 0) = 1; // Valid.
	Positions(19, 10, 1) = -31.2; // X.
	Positions(19, 10, 2) = -12; // Y.

	// Formation 21 ... play_on_m_b 
	mMaxPersist[20] = 1;
	mNumModes[20] = 0;
	mModes[20] = (TPlayMode *)malloc((int)mNumModes[20] * sizeof(TPlayMode));
	mCheckBall[20] = 1;
	mBall[20].x() = 0;
	mBall[20].y() = -34;
	mBallMin[20].x() = -52;
	mBallMin[20].y() = -5000;
	mBallMax[20].x() = 52;
	mBallMax[20].y() = 34;
	mMultiplyer[20] = 1;
	mCheckVapno[20] = 0;

	Positions(20, 0, 0) = 0; // Valid.
	Positions(20, 1, 0) = 1; // Valid.
	Positions(20, 1, 1) = 29.8; // X.
	Positions(20, 1, 2) = 0; // Y.
	Positions(20, 2, 0) = 1; // Valid.
	Positions(20, 2, 1) = 24.8; // X.
	Positions(20, 2, 2) = 17.6; // Y.
	Positions(20, 3, 0) = 1; // Valid.
	Positions(20, 3, 1) = 24.8; // X.
	Positions(20, 3, 2) = -20; // Y.
	Positions(20, 4, 0) = 1; // Valid.
	Positions(20, 4, 1) = 5; // X.
	Positions(20, 4, 2) = 20; // Y.
	Positions(20, 5, 0) = 1; // Valid.
	Positions(20, 5, 1) = 5; // X.
	Positions(20, 5, 2) = -5; // Y.
	Positions(20, 6, 0) = 1; // Valid.
	Positions(20, 6, 1) = 5; // X.
	Positions(20, 6, 2) = -29; // Y.
	Positions(20, 7, 0) = 1; // Valid.
	Positions(20, 7, 1) = -5; // X.
	Positions(20, 7, 2) = -20; // Y.
	Positions(20, 8, 0) = 1; // Valid.
	Positions(20, 8, 1) = -8; // X.
	Positions(20, 8, 2) = 1.2; // Y.
	Positions(20, 9, 0) = 1; // Valid.
	Positions(20, 9, 1) = -8; // X.
	Positions(20, 9, 2) = 14; // Y.
	Positions(20, 10, 0) = 1; // Valid.
	Positions(20, 10, 1) = -14.8; // X.
	Positions(20, 10, 2) = -13.2; // Y.

	// Formation 22 ... play_on_m_t_3 
	mMaxPersist[21] = 1;
	mNumModes[21] = 0;
	mModes[21] = (TPlayMode *)malloc((int)mNumModes[21] * sizeof(TPlayMode));
	mCheckBall[21] = 1;
	mBall[21].x() = 0;
	mBall[21].y() = 34;
	mBallMin[21].x() = -52;
	mBallMin[21].y() = -34;
	mBallMax[21].x() = 52;
	mBallMax[21].y() = 5000;
	mMultiplyer[21] = 3;
	mCheckVapno[21] = 1;

	Positions(21, 0, 0) = 0; // Valid.
	Positions(21, 1, 0) = 0; // Valid.
	Positions(21, 2, 0) = 0; // Valid.
	Positions(21, 3, 0) = 0; // Valid.
	Positions(21, 4, 0) = 0; // Valid.
	Positions(21, 5, 0) = 0; // Valid.
	Positions(21, 6, 0) = 0; // Valid.
	Positions(21, 7, 0) = 1; // Valid.
	Positions(21, 7, 1) = -33; // X.
	Positions(21, 7, 2) = -14; // Y.
	Positions(21, 8, 0) = 1; // Valid.
	Positions(21, 8, 1) = -31.2; // X.
	Positions(21, 8, 2) = 12; // Y.
	Positions(21, 9, 0) = 1; // Valid.
	Positions(21, 9, 1) = -28.8; // X.
	Positions(21, 9, 2) = 23.2; // Y.
	Positions(21, 10, 0) = 1; // Valid.
	Positions(21, 10, 1) = -31.2; // X.
	Positions(21, 10, 2) = 0; // Y.

	// Formation 23 ... play_on_m_t 
	mMaxPersist[22] = 1;
	mNumModes[22] = 0;
	mModes[22] = (TPlayMode *)malloc((int)mNumModes[22] * sizeof(TPlayMode));
	mCheckBall[22] = 1;
	mBall[22].x() = 0;
	mBall[22].y() = 34;
	mBallMin[22].x() = -52;
	mBallMin[22].y() = -34;
	mBallMax[22].x() = 52;
	mBallMax[22].y() = 5000;
	mMultiplyer[22] = 1;
	mCheckVapno[22] = 0;

	Positions(22, 0, 0) = 0; // Valid.
	Positions(22, 1, 0) = 1; // Valid.
	Positions(22, 1, 1) = 29.8; // X.
	Positions(22, 1, 2) = 0; // Y.
	Positions(22, 2, 0) = 1; // Valid.
	Positions(22, 2, 1) = 24.8; // X.
	Positions(22, 2, 2) = 20; // Y.
	Positions(22, 3, 0) = 1; // Valid.
	Positions(22, 3, 1) = 24.8; // X.
	Positions(22, 3, 2) = -17.6; // Y.
	Positions(22, 4, 0) = 1; // Valid.
	Positions(22, 4, 1) = 5; // X.
	Positions(22, 4, 2) = 29; // Y.
	Positions(22, 5, 0) = 1; // Valid.
	Positions(22, 5, 1) = 5; // X.
	Positions(22, 5, 2) = 5; // Y.
	Positions(22, 6, 0) = 1; // Valid.
	Positions(22, 6, 1) = 5; // X.
	Positions(22, 6, 2) = -20; // Y.
	Positions(22, 7, 0) = 1; // Valid.
	Positions(22, 7, 1) = -8; // X.
	Positions(22, 7, 2) = -14; // Y.
	Positions(22, 8, 0) = 1; // Valid.
	Positions(22, 8, 1) = -14.8; // X.
	Positions(22, 8, 2) = 13.2; // Y.
	Positions(22, 9, 0) = 1; // Valid.
	Positions(22, 9, 1) = -5; // X.
	Positions(22, 9, 2) = 20; // Y.
	Positions(22, 10, 0) = 1; // Valid.
	Positions(22, 10, 1) = -8; // X.
	Positions(22, 10, 2) = -1.2; // Y.

	// Formation 24 ... play_on_r_b 
	mMaxPersist[23] = 1;
	mNumModes[23] = 0;
	mModes[23] = (TPlayMode *)malloc((int)mNumModes[23] * sizeof(TPlayMode));
	mCheckBall[23] = 1;
	mBall[23].x() = 52;
	mBall[23].y() = -34;
	mBallMin[23].x() = 0;
	mBallMin[23].y() = -5000;
	mBallMax[23].x() = 5000;
	mBallMax[23].y() = 34;
	mMultiplyer[23] = 1;
	mCheckVapno[23] = 0;

	Positions(23, 0, 0) = 0; // Valid.
	Positions(23, 1, 0) = 1; // Valid.
	Positions(23, 1, 1) = 48; // X.
	Positions(23, 1, 2) = -7; // Y.
	Positions(23, 2, 0) = 1; // Valid.
	Positions(23, 2, 1) = 48; // X.
	Positions(23, 2, 2) = 8; // Y.
	Positions(23, 3, 0) = 1; // Valid.
	Positions(23, 3, 1) = 52; // X.
	Positions(23, 3, 2) = -25; // Y.
	Positions(23, 4, 0) = 1; // Valid.
	Positions(23, 4, 1) = 40.4; // X.
	Positions(23, 4, 2) = 18; // Y.
	Positions(23, 5, 0) = 1; // Valid.
	Positions(23, 5, 1) = 41; // X.
	Positions(23, 5, 2) = 0; // Y.
	Positions(23, 6, 0) = 1; // Valid.
	Positions(23, 6, 1) = 41.2; // X.
	Positions(23, 6, 2) = -22; // Y.
	Positions(23, 7, 0) = 1; // Valid.
	Positions(23, 7, 1) = 0; // X.
	Positions(23, 7, 2) = -20; // Y.
	Positions(23, 8, 0) = 1; // Valid.
	Positions(23, 8, 1) = 0; // X.
	Positions(23, 8, 2) = 1; // Y.
	Positions(23, 9, 0) = 1; // Valid.
	Positions(23, 9, 1) = 0; // X.
	Positions(23, 9, 2) = 14.4; // Y.
	Positions(23, 10, 0) = 1; // Valid.
	Positions(23, 10, 1) = 0; // X.
	Positions(23, 10, 2) = -14.4; // Y.

	// Formation 25 ... play_on_r_t 
	mMaxPersist[24] = 1;
	mNumModes[24] = 0;
	mModes[24] = (TPlayMode *)malloc((int)mNumModes[24] * sizeof(TPlayMode));
	mCheckBall[24] = 1;
	mBall[24].x() = 52;
	mBall[24].y() = 34;
	mBallMin[24].x() = 0;
	mBallMin[24].y() = -34;
	mBallMax[24].x() = 5000;
	mBallMax[24].y() = 5000;
	mMultiplyer[24] = 1;
	mCheckVapno[24] = 0;

	Positions(24, 0, 0) = 0; // Valid.
	Positions(24, 1, 0) = 1; // Valid.
	Positions(24, 1, 1) = 48; // X.
	Positions(24, 1, 2) = 7; // Y.
	Positions(24, 2, 0) = 1; // Valid.
	Positions(24, 2, 1) = 52; // X.
	Positions(24, 2, 2) = 25; // Y.
	Positions(24, 3, 0) = 1; // Valid.
	Positions(24, 3, 1) = 48; // X.
	Positions(24, 3, 2) = -8; // Y.
	Positions(24, 4, 0) = 1; // Valid.
	Positions(24, 4, 1) = 41.2; // X.
	Positions(24, 4, 2) = 22; // Y.
	Positions(24, 5, 0) = 1; // Valid.
	Positions(24, 5, 1) = 41; // X.
	Positions(24, 5, 2) = 0; // Y.
	Positions(24, 6, 0) = 1; // Valid.
	Positions(24, 6, 1) = 40.4; // X.
	Positions(24, 6, 2) = -18; // Y.
	Positions(24, 7, 0) = 1; // Valid.
	Positions(24, 7, 1) = 0; // X.
	Positions(24, 7, 2) = -14.4; // Y.
	Positions(24, 8, 0) = 1; // Valid.
	Positions(24, 8, 1) = 0; // X.
	Positions(24, 8, 2) = 14.4; // Y.
	Positions(24, 9, 0) = 1; // Valid.
	Positions(24, 9, 1) = 0; // X.
	Positions(24, 9, 2) = 20; // Y.
	Positions(24, 10, 0) = 1; // Valid.
	Positions(24, 10, 1) = 0; // X.
	Positions(24, 10, 2) = -1; // Y.

	// Formation 26 ... free_kick_l
	mMaxPersist[25] = 10;
	mNumModes[25] = 1;
	mModes[25] = (TPlayMode *)malloc((int)mNumModes[25] * sizeof(TPlayMode));
	mModes[25][0] = PM_FREE_KICK_LEFT;
	mCheckBall[25] = 1;
	mBall[25].x() = -36;
	mBall[25].y() = -20.16;
	mBallMin[25].x() = -40;
	mBallMin[25].y() = -24.16;
	mBallMax[25].x() = -32;
	mBallMax[25].y() = -16.16;
	mMultiplyer[25] = 100;
	mCheckVapno[25] = 0;

	Positions(25, 0, 0) = 0; // Valid.
	Positions(25, 1, 0) = 0; // Valid.
	Positions(25, 2, 0) = 0; // Valid.
	Positions(25, 3, 0) = 0; // Valid.
	Positions(25, 4, 0) = 0; // Valid.
	Positions(25, 5, 0) = 0; // Valid.
	Positions(25, 6, 0) = 0; // Valid.
	Positions(25, 7, 0) = 1; // Valid.
	Positions(25, 7, 1) = -36; // X.
	Positions(25, 7, 2) = -16; // Y.
	Positions(25, 8, 0) = 0; // Valid.
	Positions(25, 9, 0) = 0; // Valid.
	Positions(25, 10, 0) = 0; // Valid.

	// Formation 27 ... free_kick_l
	mMaxPersist[26] = 10;
	mNumModes[26] = 1;
	mModes[26] = (TPlayMode *)malloc((int)mNumModes[26] * sizeof(TPlayMode));
	mModes[26][0] = PM_FREE_KICK_LEFT;
	mCheckBall[26] = 1;
	mBall[26].x() = -36;
	mBall[26].y() = 20.16;
	mBallMin[26].x() = -40;
	mBallMin[26].y() = 16.16;
	mBallMax[26].x() = -32;
	mBallMax[26].y() = 24.16;
	mMultiplyer[26] = 100;
	mCheckVapno[26] = 0;

	Positions(26, 0, 0) = 0; // Valid.
	Positions(26, 1, 0) = 0; // Valid.
	Positions(26, 2, 0) = 0; // Valid.
	Positions(26, 3, 0) = 0; // Valid.
	Positions(26, 4, 0) = 0; // Valid.
	Positions(26, 5, 0) = 0; // Valid.
	Positions(26, 6, 0) = 0; // Valid.
	Positions(26, 7, 0) = 0; // Valid.
	Positions(26, 8, 0) = 0; // Valid.
	Positions(26, 9, 0) = 1; // Valid.
	Positions(26, 9, 1) = -36; // X.
	Positions(26, 9, 2) = 16; // Y.
	Positions(26, 10, 0) = 0; // Valid.
}

/*
 * Interface.
 */
// 'num' - cislo hraca (0 .. 10)
bool Formations::GetPosition(int num, Vector3f &position)
{
	if (mHistory == NULL || num < 0 || num > 10 || mNumFormationStates == 0)
		return false;

	unsigned i;
	double x = 0;
	double y = 0;

	FillRatings(num);
	if (mSumRating < 0.4)
		return false;

	for (i = 0; i < mNumFormationStates; i++)
	{
		if (Positions(i, num, 0))
		{
			x += Positions(i, num, 1) * mRating[i];
			y += Positions(i, num, 2) * mRating[i];
		} // if (Positions(i, num, 0))
	} // for (i = 0; i < mNumFormationStates; i++)

	position.x() = x / mSumRating;
	position.y() = y / mSumRating;

	return true;
}

void Formations::FillRatings(int num)
{
	if (mHistory == NULL)
		return;

	WorldModel wm;
	AbsBall ballPosition;

	wm = mHistory->history[0];
	if (!wm.IsValid())
		return;
		
	ballPosition = wm.ball;

	unsigned i, j;

	mSumRating = 0;
	for (i = 0; i < mNumFormationStates; i++)
	{
		mRating[i] = 0;
		if (mMaxPersist[i] > 0)
		{
			for (j = 0; j < mNumModes[i]; j++)
			{
				if (wm.playMode == mModes[i][j])
					mPersist[i] = mMaxPersist[i];
			} // for (j = 0; j < mNumModes[i]; j++)
			if (!mNumModes[i])
				mPersist[i] = mMaxPersist[i];
		} // if (mMaxPersist[i] > 0)
		if (Positions(i, num, 0) && (mPersist[i] != 0) || (mMaxPersist[i] <= 0))
		{
			mRating[i] = 1;
			if (mMaxPersist[i] > 0)
				mRating[i] *= mPersist[i] / mMaxPersist[i];
			if (mCheckBall[i])
				mRating[i] *= CheckBallPos(ballPosition, mBall[i], mBallMin[i], mBallMax[i]);
			/* DOROBIT
			if (mCheckVapno[i])
				mRating[i] *= myKernel->CountEnemiesInRect(-52.5, 20.16, -36, -20.16);
			*/
			mRating[i] *= mMultiplyer[i];
			mSumRating += mRating[i];
		} // if (Positions(i, num, 0) && (mPersist[i] != 0) || (mMaxPersist[i] <= 0)))
	} // for (i = 0; i < mNumFormationStates; i++)
}

void Formations::SetTime(unsigned time)
{
	if (mHistory == NULL)
		return;

	unsigned i;
	Vector3f temp;
	int myNum = 0;
	WorldModel wm;
	
	wm = mHistory->history[0];
	if (!wm.IsValid())
		return;
	
	if (mNow == time)
		mNowRepeat++;
	else
	{
		mNowRepeat = 0;
		mNow = time;
	} // else

	for (i = 0; i < mNumFormationStates; i++)
	{
		if (mPersist[i] > 0)
			mPersist[i] = mPersist[i] - 1;
	} // for (i = 0; i < mNumFormationStates; i++)

	if ((myNum <= 0) ||(myNum > 11))
		myNum = mHistory->teamInfo.mTeamUnum;
	if (GetPosition(myNum - 1, temp))
		wm.myself.mPosition = temp;
}

double Formations::CheckBallPos(AbsBall currentBall, Vector3f ball, Vector3f ballMin, Vector3f ballMax)
{
	bool bReturn;
	
	bReturn = CheckBallPos(currentBall.mPosition, ball, ballMin, ballMax);
	
	return bReturn;
}

double Formations::CheckBallPos(Vector3f currentBall, Vector3f ball, Vector3f ballMin, Vector3f ballMax)
{
	double res = 0;

	currentBall -= ball;
	ballMin -= ball;
	ballMax -= ball;

	if (currentBall.x() >= 0)
	{
		currentBall.x() /= ballMax.x();
		currentBall.x() *= currentBall.x();
		res += currentBall.x();
	} // if (currentBall.x() >= 0)
	else
	{
		currentBall.x() /= ballMin.x();
		currentBall.x() *= currentBall.x();
		res += currentBall.x();
	} // else

	if (currentBall.y() >= 0)
	{
		currentBall.y() /= ballMax.y();
		currentBall.y() *= currentBall.y();
		res += currentBall.y();
	} // if (currentBall.y() >= 0)
	else
	{
		currentBall.y() /= ballMin.y();
		currentBall.y() *= currentBall.y();
		res += currentBall.y();
	} // else

	res = gSqrt(res);
	if (res < 0)
		return 0; // Toto by sa nemalo stat...
	else if (res > 1)
		res = 1;

	return 1 - res;
}
