#ifndef FORMATIONS_H
#define FORMATIONS_H

#include <zeitgeist/class.h>
#include "soccer/soccertypes.h"
#include "../BaseModule.h"
#include "PrimaryISModule.h"
#include "history.h"

using namespace salt;

class Formations : public BaseISModule
{
protected:
	unsigned *mMaxPersist;
	double *mMultiplyer;
	unsigned *mNumModes;
	TPlayMode **mModes;
	unsigned *mCheckBall;
	Vector3f *mBall;
	Vector3f *mBallMin;
	Vector3f *mBallMax;
	unsigned *mCheckVapno;

	double *mPositions;

	double *mRating;
	unsigned *mPersist;

	double mSumRating;

	unsigned mNow;
	unsigned mNowRepeat;
	unsigned mNumFormationStates;
	unsigned mNumPlayers;

	// Internal memory management.
	void AllocateStorage(unsigned numFormationStates, unsigned numPlayers);
	void DeallocateStorage();

	// Indexer.
	double &Positions(unsigned formationState, unsigned player, unsigned value);
	
	// Pointer to History modul.
	shared_ptr<History> mHistory;

	// Pointer to ISModuleHandler.
	shared_ptr<ISModuleHandler> mISModuleHandler;
	
public:
	// Constructor/destructor
	Formations();
	~Formations();

	// Overrided virtuals.
	void OnLink();
	void OnUnLink();

	// Initializations.
	void LoadDefaultFormations();

	// Interface.
	bool GetPosition(int num, Vector3f &position);
	void SetTime(unsigned time);
	void FillRatings(int num);
	double CheckBallPos(AbsBall currentBall, Vector3f ball, Vector3f ballMin, Vector3f ballMax);
	double CheckBallPos(Vector3f currentBall, Vector3f ball, Vector3f ballMin, Vector3f ballMax);
};

DECLARE_CLASS(Formations);

#endif FORMATIONS_H
