#include "Predict.h"

/*
 * Constructor/destructor.
 */
Predict::Predict()
{
}

Predict::~Predict()
{
}

/*
 * Overrided virtuals.
 */
void Predict::OnLink()
{
	// Ziskaj ukazovatel do stromu IS modulov.
	mISModuleHandler = shared_dynamic_cast<ISModuleHandler> (GetCore()->Get("/Agent/Sys/ISModuleHandler"));
	if (mISModuleHandler.get() == 0)
		GetLog()->Error() << "ERROR: (Predict) cannot find ISModuleHandler\n";
		
	// Registruj tento modul.
	mISModuleHandler->InsertModule(this);
	
	// Ziskaj ukazovatel na History modul.
	mHistory = shared_dynamic_cast<History> (GetCore()->Get("/Agent/Sys/History"));
	if (mHistory.get() == 0)
		GetLog()->Error() << "ERROR: (Predict) cannot find History module.";
}

void Predict::OnUnLink()
{
	mISModuleHandler.reset();
	mHistory.reset();
}

/*
 * WM/time interface.
 */
// 'indexShift' is valid for values (-(HISTORY_LENGTH - 1), 0>
WorldModel Predict::GetWorldModel(int timeShift)
{
	if (mHistory == NULL || timeShift > 0 ||
		(timeShift = abs((int)timeShift)) >= HISTORY_LENGTH)
	{
		WorldModel wm;
		
		// Vrat prazdny (invalidny) WorldModel.
		return wm;
	} // if (timeShift > 0 || timeShift >= HISTORY_LENGTH)
	
	return mHistory->history[timeShift];
}

bool Predict::PredictBallPosition(int timeForward, Vector3f &output, float &predictionProbability)
{
	predictionProbability = 0;

	return false;

	/*
	Vector3f probableBallPosition;
	Vector3f currentBallPosition;
	Vector3f probablePlayerPosition;
	Vector3f temporary1, temporary2;
	TTeamIndex ourTeam, theirTeam;
	float distanceBallFromPlayer, minimalKickDistance;
	
	// Ziskaj pravdepodobnu poziciu lopty.
	probableBallPosition = SimplePredictBallPosition(numStepsForward)
	predictionProbability = 100;
	
	// Zohladni vplyv nasich hracov.
	ourTeam = worldModel->GetMyTeam();
	currentBallPosition = GetCurrentBallPosition();
	minimalKickDistance = worldModel->GetMinimalKickDistance();
	foreach (ourPlayer)
	{
		// Zisti, kde sa bude nachadzat hrac.
		probablePlayerPosition = SimplePredictPlayerPosition(numStepsForward, ourTeam, ourPlayer);

		// Zisti smer pohybu hraca a smer hraca k lopte.
		temporary1 = probablePlayerPosition;
		temporary1.Normalize();
		temporary2 = GetCurrentPlayerPosition(ourTeam, ourPlayer) - currentBallPosition;
		temporary2.Noramlize();
	
		// Ak sa hrac pohybuje smerom k lopte ...
		if (temporary1 == temporary2)
		{
			// Zisti vzdialenost hraca od lopty v buducnosti.
			temporary1 = probableBallPosition - probablePlayerPosition;
			distanceBallFromPlayer = temporary1.Length();

			// Ak sa hrac v danom case dostane k lopte, zniz pravdepodobnost na polovicu,
			// pretoze koli jednoduchosti neuvazujem, ze by som menil smerom lopty, ale predpokladam,
			// ze lopta uz idem spravnym smerom a nasim hracom bude iba postrcena dalej.
			if (distanceBallFromPlayer < minimalKickDistance)
				predictionProbability /= 2;
		} // if (temporary1 == temporary2)
	} // foreach (ourPlayer)
   
	// Zohladni vplyv protivnikovych hracov.
	theirTeam = ...;
	foreach (theirPlayer)
	{
		// Zisti, kde sa bude nachadzat hrac.
		probablePlayerPosition = SimplePredictPlayerPosition(numStepsForward, theirTeam, theirPlayer);

		// Zisti smer pohybu hraca a smer hraca k lopte.
		temporary1 = probablePlayerPosition;
		temporary1.Normalize();
		temporary2 = GetCurrentPlayerPosition(theirTeam, theirPlayer) - currentBallPosition;
		temporary2.Noramlize();

		// Ak sa hrac pohybuje smerom k lopte ...
		if (temporary1 == temporary2)
		{
			// Zisti vzdialenost hraca od lopty v buducnosti.
			temporary1 = probableBallPosition - probablePlayerPosition;
			distanceBallFromPlayer = temporary1.Length();

			// Ak sa hrac v danom case dostane k lopte, predpokladaj, ze
			// uplne zmeni smer lopty a tak je predpoved irelevantna.
			if (distanceBallFromPlayer < minimalKickDistance)
			{
				predictionProbability = 0;
		
				break; // foreach (theirPlayer)
			} // if (distanceBallFromPlayer < minimalKickDistance)
		} // if (temporary1 == temporary2)
	} // foreach (theirPlayer)
	
	// Vrat predpokladanu hodnotu.
	return probableBallPosition;
	*/
}

/*
 * Player interface.
 */
bool Predict::GetPlayerPosition(int time, TTeamIndex team, int player, Vector3f &output)
{
	if (time <= 0)
	{
		WorldModel wm;

		wm = GetWorldModel(time);
		if (wm.IsValid())
		{
			if (team == mHistory->teamInfo.mTeamIndex)
				output = wm.friends[player].mPosition;
			else
				output = wm.enemies[player].mPosition;
				
			return true;
		} // if (wm.IsValid())
		else
			return false;
	} // if (time == 0)
	else
		return SimplePredictPlayerPosition(time, team, player, output);
}

bool Predict::SimplePredictPlayerPosition(int timeForward, TTeamIndex team, int player, Vector3f &output)
{
	Vector3f temp0, temp1;
   
	// Ziskaj smer pohybu hraca.
	if (!GetPlayerPosition(0, team, player, temp0))
		return false;
	if (!GetPlayerPosition(1, team, player, temp1))
		return false;
		
	output = temp1 - temp0;
	
	// Predpokladaj poziciu o 'timeForward' krokov.
	// Toto je pozicia pri konstantnej rychlosti a bez vplyvu ostatnych hracov.
	output *= timeForward;
	
	return true;

	/*
	Vector3f playerDirection;
   
  	// Ziskaj smer pohybu hraca a normalizuj ho.
  	playerDirection = GetPlayerPosition(1, team, player) - GetPlayerPosition(0, team, player);
  	playerDirection.Normalize();
   
  	// Predpokladaj poziciu o 'timeForward' krokov.
  	// Toto je pozicia pri konstantnej rychlosti a bez vplyvu ostatnych hracov a lopty.
  	playerDirection *= timeForward;
   
  	// Zohladni spomalenie hraca vzhladom na jeho energiu (nedorobene).
  	...
   
  	// Vrat predpokladanu hodnotu.
  	return playerDirection;
	*/
}

bool Predict::PredictPlayerPosition(int timeForward, TTeamIndex team, int player, Vector3f &output, float &predictionProbability)
{
	predictionProbability = 0;

	return false;
}

bool Predict::GetVisionObjectPosition(int time, Parser::VisionObject visionObject, Vector3f &output)
{
	if (time <= 0)
	{
		WorldModel wm;

		wm = GetWorldModel(time);
		if (wm.IsValid())
		{
			output = wm.flagsAndGoals[visionObject].mPosition;
			
			return true;
		} // if (wm.IsValid())
		else
			return false;
	} // if (time == 0)
	else
		return SimplePredictVisionObjectPosition(time, visionObject, output);
}

bool Predict::SimplePredictVisionObjectPosition(int timeForward, Parser::VisionObject visionObject, Vector3f &output)
{
	Vector3f temp0, temp1;
   
	// Ziskaj smer pohybu visualneho objektu.
	if (!GetVisionObjectPosition(0, visionObject, temp0))
		return false;
	if (!GetVisionObjectPosition(1, visionObject, temp1))
		return false;
		
	output = temp1 - temp0;
	
	// Predpokladaj poziciu o 'timeForward' krokov.
	// Toto je pozicia pri konstantnej rychlosti a bez vplyvu ostatnych hracov.
	output *= timeForward;
	
	return true;
}

bool Predict::PredictVisionObjectPosition(int timeForward, Parser::VisionObject visionObject, Vector3f &output, float &predictionProbability)
{
	predictionProbability = gMax(0.0f, 100.0f - timeForward * 10.0f);
	
	return SimplePredictVisionObjectPosition(timeForward, visionObject, output);
}

int Predict::GetNumStepsForMeetingOfPlayerAndBall(TTeamIndex team, int player, Vector3f &direction)
{
	Vector3f playerCurrentPosition, playerLastPosition;
	
	// Zisti poziciu hraca v predchadzajucom a sucasnom kroku.
	// Priemernu rychlost (velkost) pohybu ziskame ako dlzku vektora medzi tymito dvoma bodmi.
	if (!GetPlayerPosition(team, player, playerCurrentPosition))
		return -1;
	if (!GetPlayerPosition(1, team, player, playerLastPosition))
		return -1;

	return GetNumStepsForMeetingOfPlayerAndBall(team, player, direction, (playerLastPosition - playerCurrentPosition).Length(), MAXINT);
}

int Predict::GetNumStepsForMeetingOfPlayerAndBall(TTeamIndex team, int player, Vector3f &direction, float averagePlayerSpeed, int maxSteps)
{
	int i;
	float distancePlayerBall, ballTrackLength, playerTrackLength;
	Vector3f ballPosition, playerPosition, playerTempPosition, tempVector;
	bool found;
	
	// Ziskaj aktualnu poziciu hraca.
	if (!GetPlayerPosition(team, player, playerPosition))
		return -1;
	// Ziskaj aktualnu poziciu lopty.
	if (!GetBallPosition(ballPosition))
		return -1;
	// Ziskaj poziciu lopty z predchadzajuceho kroku.
	if (!GetBallPosition(1, tempVector))
		return -1;
	
	// Vypocitaj vzdialenost aktualnych pozicii hraca a lopty.
	// Na zistenie, ci sme nasli poziciu, kam sa v danom case dostane lopta aj hrac
	// pouzijeme jednoduchy trojuholnik (sucet dlzok odvesien musi byt vacsi ako je dlzka prepony,
	// inak to nie je trojuholnik, a teda sa v tom bode nemozu oba objekty nachadzat v danom
	// case sucasne).
	distancePlayerBall = (playerPosition - ballPosition).Length();
	
	// Preved poziciu lopty na smerovy vektor aktualneho pohybu lopty.
	ballPosition = tempVector - ballPosition;
	
	found = false;
	i = 0;
	do
	{
		// Vypocitaj dlzku vektora (drahy) lopty v danom kroku (case).
		ballTrackLength = (ballPosition * (i + 1)).Length();
		
		// Ak je to trojuholnik, nasli sme riesenie ...
		if (distancePlayerBall > ballTrackLength + averagePlayerSpeed)
			found = true;
		// ... inak pokracuj v hladani
		else
			i++;
	} while (!found && i <= maxSteps); // Hladanie ukonci, ak sme nasli vysledok alebo sme prekrocili maximalny pocet krokov.
	
	if (found)
	{
		// Vytvor normalizovany smerovy vektor od hraca k predpokladanej pozicii
		// lopty v case (kroku) i.
		direction = (ballPosition * (i + 1)) - playerPosition;
		direction.Normalize();
		
		// Vrat predpokladany pocet krokov, po ktorych sa pri pohybe v smere 'direction'
		// stretne hrac s loptou.
		return i;
	} // if (found)
	else
		return -1;
}