#ifndef PREDICT_H
#define PREDICT_H

#include <zeitgeist/class.h>
#include "../BaseModule.h"
#include "PrimaryISModule.h"
#include "history.h"
#include "../parser/parser.h"

using namespace salt;

#define MAXINT 32767

class Predict : public BaseISModule
{
protected:
	// Pointer to History modul.
	shared_ptr<History> mHistory;
	
	// Pointer to ISModuleHandler.
	shared_ptr<ISModuleHandler> mISModuleHandler;

public:
	// Constructor/destructor.
	Predict();
	virtual ~Predict();

	// Overrided virtuals.
	void OnLink();
	void OnUnLink();

	// WM/time interface.
	WorldModel GetWorldModel(int timeShift);

	// Ball interface.
	bool GetBallPosition(/*out*/ Vector3f &output) { return GetBallPosition(0, output); }
	bool GetBallPosition(int time, /*out*/ Vector3f &output) { return GetVisionObjectPosition(Parser::VO_BALL, output); }
	bool SimplePredictBallPosition(int timeForward, /*out*/ Vector3f &output) { return SimplePredictVisionObjectPosition(timeForward, Parser::VO_BALL, output); }
	bool PredictBallPosition(int timeForward, /*out*/ Vector3f &output, /*out*/ float &predictionProbability); // Tato bude asi potrebovat viac logiky vyriesit, ako ine visualne objekty!

	// Player interface.
	bool GetPlayerPosition(TTeamIndex team, int player, /*out*/ Vector3f &output) { return GetPlayerPosition(0, team, player, output); }
	bool GetPlayerPosition(int time, TTeamIndex team, int player, /*out*/ Vector3f &output);
	bool SimplePredictPlayerPosition(int timeForward, TTeamIndex team, int player, /*out*/ Vector3f &output);
	bool PredictPlayerPosition(int timeForward, TTeamIndex team, int player, /*out*/ Vector3f &output, /*out*/ float &predictProbability);
	
	// VisionObject interface.
	bool GetVisionObjectPosition(Parser::VisionObject visionObject, /*out*/ Vector3f &output) { return GetVisionObjectPosition(0, visionObject, output); }
	bool GetVisionObjectPosition(int time, Parser::VisionObject visionObject, /*out*/ Vector3f &output);
	bool SimplePredictVisionObjectPosition(int timeForward, Parser::VisionObject visionObject, /*out*/ Vector3f &output);
	bool PredictVisionObjectPosition(int timeForward, Parser::VisionObject visionObject, /*out*/ Vector3f &output, float &predictProbability);
	
	int GetNumStepsForMeetingOfPlayerAndBall(TTeamIndex team, int player, /*out*/ Vector3f &direction);
	int GetNumStepsForMeetingOfPlayerAndBall(TTeamIndex team, int player, /*out*/ Vector3f &direction, float averagePlayerSpeed) { return GetNumStepsForMeetingOfPlayerAndBall(team, player, direction, averagePlayerSpeed, MAXINT); }
	int GetNumStepsForMeetingOfPlayerAndBall(TTeamIndex team, int player, /*out*/ Vector3f &direction, float averagePlayerSpeed, int maxSteps);
};

DECLARE_CLASS(Predict);

#endif PREDICT_H