#ifndef HISTORY_H
#define HISTORY_H

#include <zeitgeist/leaf.h>
#include <zeitgeist/class.h>
#include <zeitgeist/zeitgeist.h>
#include <soccer/soccertypes.h>
#include "PrimaryISModule.h"
#include <string>
#include <list> 

using namespace std;
using namespace boost;
using namespace zeitgeist;
using namespace salt;

#define HISTORY_LENGTH 20

// absolutne pozicie hraca
struct AbsPlayer
{
    bool valid;		//pouzite na urcenie ci je zaznam platny
		Vector3f mPosition;
};

// nehybajuci sa objekt - t.j. branky a rohove vlajky
struct StaticObject
{
    int objectType;	// vision object (z parser.h)
    bool valid;		//pouzite na urcenie ci je zaznam platny    
		Vector3f mPosition;	
};

// absolne pozicie lopty
struct AbsBall
{
    bool valid;		//pouzite na urcenie ci je zaznam platny
		Vector3f mPosition;		
};

struct WorldModel
{
    AbsPlayer myself;
    AbsPlayer friends[11];
    AbsPlayer enemies[11];
    StaticObject flagsAndGoals[8];
    AbsBall ball;
    TPlayMode playMode;

    WorldModel()
    {
			Invalidate();
    }
		
		void Invalidate()
		{
			unsigned i;
			
			myself.valid = false;
			for (i = 0; i < 11; i++)
			{
				friends[i].valid = false;
				enemies[i].valid = false;
				if (i < 8)
					flagsAndGoals[i].valid = false;
			} // for (i = 0; i < 11; i++)
			ball.valid = false;
			playMode = PM_NONE;
		}
		
		bool IsValid()
		{
			unsigned i;
			bool bValid;
			
			bValid = myself.valid;
			for (i = 0; i < 11; i++)
			{
				// Optimalizuj cyklus. Ak raz nie je spravny, tak rovno skonci.
				if (!bValid)
					return false;
					
				bValid &= friends[i].valid;
				bValid &= enemies[i].valid;
				if (i < 8)
					bValid &= flagsAndGoals[i].valid;
			} // for (i = 0; i < 11; i++)
			bValid &= ball.valid;
			bValid &= playMode != PM_NONE;
			
			return bValid;
		}
};

class History : public BaseISModule
{
private:

    shared_ptr<ISModuleHandler> mISModuleHandler;
    shared_ptr<PrimaryISModule> mPrimaryISModule;

public:
    PlayingField myField;	// struktury z PrimaryISModule
    Agent agentParameters;	// struktury z PrimaryISModule
    Ball ballParameters;	// struktury z PrimaryISModule
    TeamInfo teamInfo;		// struktury z PrimaryISModule
    const int history_length;
    
    std::vector<WorldModel> history;
 
    History();
    void OnLink();
    void OnUnLink();
    bool DataChanged(int);
    WorldModel CreateWorldModel();
    WorldModel GetData(int);
};

DECLARE_CLASS(History);

#endif