#include "invoker.h"

Invoker::Invoker(Zeitgeist * zg) :
	mZeitgeist(NULL),
	mInitialized(false)
{
	mZeitgeist = zg;
	Init();
}

Invoker::~Invoker()
{

}

void Invoker::Run()
{
    bool run = true;
    int sensationCount = 0;
    if (!mInitialized) return;
	
    printf("Agent_Hazard started\n");

    while (run)
    {
        if (! mCommServer->GetInput()) continue;
	
	// 4.6.2006 matell
 	mParser->Parse(mCommServer->GetMsg());
	mISModuleHandler->NotifyModules(0);
	
	cout << "Msg from server: " << mCommServer->GetMsg() << "\n";
	fflush(0);

        switch(mCommServer->GetMsg()[0])
        {
        case 'D' :
		mWrapper->DoneInitMessage();
		break;

        case 'K' :
		mWrapper->DoneThinkingMessage();
		break;

        case 'S' :
		switch (sensationCount)
		{
			case 0:
				mWrapper->CreateAgent();
				break;
			
			case 1:
			case 2:
			case 4:
			case 5:
				break;
			
			case 3:
				mWrapper->InitAgent("0", mPlayer->getTeamName());
				mParser->mPrimaryISModule->SetTeamName(mPlayer->getTeamName());
				break;
				
			default:
				mPlayer->Behave();
				break;
		}

    		++sensationCount;
		mWrapper->DoneThinkingMessage();
		break;
	
	case 'X' :
		run = false; // terminate agent
		printf("Agent_Hazard terminated\n");
            break;

        default:
            printf("received unknown data: ");
            printf(mCommServer->GetMsg());
            printf("\n");
            break;
        }
    }
	
}

void Invoker::Init()
{
   // register agent classes
	shared_ptr<CoreContext> context = mZeitgeist->CreateContext();
    
	//	iportneme sexparser
	mZeitgeist->GetCore()->ImportBundle("sexpparser");
    
	//	nahodime do zeitgeistu commserver, parser a wrapper
	if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(CommServer), "/Agent_Hazard/Core/"))
		mCommServer = shared_static_cast<CommServer> (context->New("/Agent_Hazard/Core/CommServer", "/Agent/Sys/CommServer"));

	if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(Wrapper), "/Agent_Hazard/Core/"))
		mWrapper = shared_static_cast<Wrapper> (context->New("/Agent_Hazard/Core/Wrapper", "/Agent/Sys/Wrapper"));

	if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(ISModuleHandler),"/Agent_Hazard/Core/"))
		mISModuleHandler = shared_static_cast<ISModuleHandler> (context->New("/Agent_Hazard/Core/ISModuleHandler", "/Agent/Sys/ISModuleHandler"));
	
	if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(PrimaryISModule),"/Agent_Hazard/Core/"))
	    mPrimaryISModule = shared_static_cast<PrimaryISModule> (context->New("/Agent_Hazard/Core/PrimaryISModule", "/Agent/Sys/PrimaryISModule"));
	
	/*if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(History),"/Agent_Hazard/Core/"))
	    mHistory = shared_static_cast<History> (context->New("/Agent_Hazard/Core/History", "/Agent/Sys/History"));*/
	
	if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(Parser), "/Agent_Hazard/Core/"))
		mParser = shared_static_cast<Parser> (context->New("/Agent_Hazard/Core/Parser", "/Agent/Sys/Parser"));

	printf("Invoker::Init() : Inicializacia\n");
	
	if (!mZeitgeist->GetCore()->GetScriptServer()->Run("agent.rb"))
	    printf("*	Nezbehol init script\n");

//	if (mZeitgeist->GetCore()->RegisterClassObject(new CLASS(Player),"/Agent/Core/"))
//	    mPlayer = shared_dynamic_cast<Player> (context->New("/Agent/Core/HazardPlayer", "/Agent/Modules/Player"));

	mHistory = shared_static_cast<History> (context->Get("/Agent/Modules/History"));
	mPlayer = shared_static_cast<Player> (context->Get("/Agent/Modules/Player"));

	printf("*	Comm server: %d\n", mCommServer.get());
	printf("*	Parser: %d\n", mParser.get());
	printf("*	Wrapper: %d\n", mWrapper.get());
	printf("*	IS Module Handler: %d\n*\n", mISModuleHandler.get());
	
	printf("*	PrimaryISModule: %d\n",mPrimaryISModule.get());
	printf("*	History: %d\n",mHistory.get());
	printf("*	Player: %d\n", mPlayer.get());

	mInitialized = ((mCommServer.get()) && (mParser.get()) && (mWrapper.get()) && (mISModuleHandler.get()) && (mPrimaryISModule.get()) &&(mHistory.get()) && (mPlayer.get()));
}
