#include "parser.h"

using namespace zeitgeist;
using namespace std;
using namespace boost;
using namespace oxygen;
using namespace salt;

Parser::Parser()
{   
/*    mPrimaryISModule2 = shared_dynamic_cast<PrimaryISModule>(GetCore()->Get("/Agent/Sys/PrimaryISModule"));
    if(mPrimaryISModule.get() == 0)
    {
	cout << "Nemame PrimaryISModule" << endl;
	usable = false;
    }
    else
    mPrimaryISModule = new PrimaryISModule();*/
     usable = true;
}

Parser::~Parser()
{
}


void
Parser::SetupGameStateMap()
{
    mPlayModeMap.clear();

    mPlayModeMap[STR_PM_BeforeKickOff] = PM_BeforeKickOff;
    mPlayModeMap[STR_PM_KickOff_Left]  = PM_KickOff_Left;
    mPlayModeMap[STR_PM_KickOff_Right] = PM_KickOff_Right;
    mPlayModeMap[STR_PM_PlayOn]        = PM_PlayOn;
    mPlayModeMap[STR_PM_KickIn_Left]   = PM_KickIn_Left;
    mPlayModeMap[STR_PM_KickIn_Right]  = PM_KickIn_Right;
    mPlayModeMap[STR_PM_Goal_Left]     = PM_Goal_Left;
    mPlayModeMap[STR_PM_Goal_Right]    = PM_Goal_Right;
    mPlayModeMap[STR_PM_GameOver]      = PM_GameOver;
}

void
Parser::SetupVisionObjectMap()
{
    mVisionObjectMap.clear();

    // <ObjectName>+<Id> -> VisionObject
    mVisionObjectMap["Flag1_l"] = VO_FLAG1L;
    mVisionObjectMap["Flag1_r"] = VO_FLAG1R;
    mVisionObjectMap["Flag2_l"] = VO_FLAG2L;
    mVisionObjectMap["Flag2_r"] = VO_FLAG2R;
    mVisionObjectMap["Goal1_l"] = VO_GOAL1L;
    mVisionObjectMap["Goal1_r"] = VO_GOAL1R;
    mVisionObjectMap["Goal2_l"] = VO_GOAL2L;
    mVisionObjectMap["Goal2_r"] = VO_GOAL2R;
    mVisionObjectMap["Ball"]    = VO_BALL;
}

bool
Parser::ConstructInternal()
{
    mParser = shared_dynamic_cast<BaseParser>(GetCore()->New("SexpParser"));
    if (mParser.get() == 0)
        {

            return false;
        }

    SetupGameStateMap();
    SetupVisionObjectMap();
    
    mPlayerVisionObjectMap.clear(); // aldur 15.4.2006 inicializacia hash tabulky pre hracov

    return true;
}

// 11-04-06 matell
void
Parser::ParseBallPosition()
{
    VisionSense ballSense =
    GetVisionSense(Parser::VO_BALL);
    
    mPrimaryISModule->mBall.mDistance = ballSense.distance;  
    mPrimaryISModule->mBall.mTheta = ballSense.theta;
    mPrimaryISModule->mBall.mPhi = ballSense.phi;
}

// 11-04-06 matell
void
Parser::ParseL1Position()
{
    VisionSense L1 =
    GetVisionSense(Parser::VO_FLAG1L);
    
    mPrimaryISModule->mNonMovableObjects.mL1Distance = L1.distance;  
    mPrimaryISModule->mNonMovableObjects.mL1Theta = L1.theta;
    mPrimaryISModule->mNonMovableObjects.mL1Phi = L1.phi;
}

void
Parser::ParseTeamIndex(const Predicate& predicate)
{
    string team;
    if (! predicate.GetValue(predicate.begin(),"team",team))
        {
            return;
        }

    // parse team index
    if (team == "left")
        mPrimaryISModule->mTeamInfo.mTeamIndex = TI_LEFT;
    else if (team == "right")
        mPrimaryISModule->mTeamInfo.mTeamIndex = TI_RIGHT;
    else
        {
            mPrimaryISModule->mTeamInfo.mTeamIndex = TI_NONE;
        }
}

void Parser::ParsePlayMode(const Predicate& predicate)
{
    string mode;
    if (! predicate.GetValue(predicate.begin(),"playmode", mode))
        {
            return;
        }

    // parse play mode
    TPlayModeMap::iterator iter = mPlayModeMap.find(mode);
    if (iter != mPlayModeMap.end())
        {		
		//     mPlayModeMap[STR_PM_BeforeKickOff] = PM_BeforeKickOff;
		//cout << endl << "mtime " << mTime << endl;
            mPrimaryISModule->mPlayMode = (TPlayMode)(*iter).second;
        } 
}

void Parser::ParseGameState(const Predicate& predicate)
{
    ParseTeamIndex(predicate);
    ParsePlayMode(predicate);
    predicate.GetValue(predicate.begin(),"FieldLength",   mPrimaryISModule->mField.mFieldLength);
    predicate.GetValue(predicate.begin(),"FieldWidth",    mPrimaryISModule->mField.mFieldWidth);
    predicate.GetValue(predicate.begin(),"FieldHeigth",   mPrimaryISModule->mField.mFieldHeight);
    predicate.GetValue(predicate.begin(),"GoalWidth",     mPrimaryISModule->mField.mGoalWidth);
    predicate.GetValue(predicate.begin(),"GoalDepth",     mPrimaryISModule->mField.mGoalDepth);
    predicate.GetValue(predicate.begin(),"GoalHeight",    mPrimaryISModule->mField.mGoalHeight);
    predicate.GetValue(predicate.begin(),"BorderSize",    mPrimaryISModule->mField.mBorderSize);
    predicate.GetValue(predicate.begin(),"time",          mTime);
    predicate.GetValue(predicate.begin(),"unum",          mPrimaryISModule->mTeamInfo.mTeamUnum);
    predicate.GetValue(predicate.begin(),"AgentRadius",   mPrimaryISModule->mAgent.mAgentRadius);
    predicate.GetValue(predicate.begin(),"AgentMass",     mPrimaryISModule->mAgent.mAgentMass);
    predicate.GetValue(predicate.begin(),"AgentMaxSpeed", mPrimaryISModule->mAgent.mAgentMaxSpeed);
    predicate.GetValue(predicate.begin(),"BallRadius",    mPrimaryISModule->mBall.mBallRadius);
    predicate.GetValue(predicate.begin(),"BallMass",      mPrimaryISModule->mBall.mBallMass);
    
    ParseBallPosition();
    ParseL1Position();	// relativna poloha hraca voci bodu L1
    
    //mPrimaryISModule->PrintAllStructs();    // testovaci vypis
    
}

void Parser::ParseObjectVision(const Predicate& predicate)
{
    for (
         Predicate::Iterator iter(predicate);
         iter != iter.end();
         ++iter
         )
        {
            // extract the element as a parameter list
            Predicate::Iterator paramIter = iter;
            if (! predicate.DescentList(paramIter))
                {
                    continue;
                }

            // read the object name
            string name;
            if (! predicate.GetValue(paramIter,name))
            {
                continue;
            }
	    // aldur 15.4.2006 treba specialne spracovanie pre hracov
	    if(name == "Player")
	    {
		PolPlayer playerSense;
		string s;
		
		
		predicate.GetValue(paramIter,"team",s);
		
		playerSense.team = s;
		
		predicate.GetValue(paramIter,"id",s);
		
		int id = std::atoi(s.c_str());
		playerSense.id = id;
		
		Predicate::Iterator polIter = paramIter;
        	if (! predicate.FindParameter(polIter,"pol"))
                {
                    continue;
                }

        	// read the position vector
    	        if (
            	    (! predicate.AdvanceValue(polIter,playerSense.distance)) ||
            	    (! predicate.AdvanceValue(polIter,playerSense.theta)) ||
            	    (! predicate.AdvanceValue(polIter,playerSense.phi))
            	    )
        	{
            	    continue;
        	}
		
		mPrimaryISModule->players.insert(mPrimaryISModule->players.begin(),playerSense);
		/*
		cout << "Hrac timu : " << playerSense.team << " ID: "<< playerSense.id 
		     << " Vzdialenost: " << playerSense.distance  
		     << " Phi: "<< playerSense.phi << " Theta: "<< playerSense.theta <<endl; 
				 */
	    }
	    else
	    {
        	// try read the 'id' section
	        string strId;
    	        if (predicate.GetValue(paramIter,"id", strId))
        	{
            	    name += strId;
        	}
		// cout << "Meno na vyhladanie: " << name << endl; // aldur 15.4.2006 
        	// try to lookup the VisionObject
        	TVisionObjectMap::iterator iter = mVisionObjectMap.find(name);
        	if (iter == mVisionObjectMap.end())
                {
                    continue;
                }

        	VisionObject vo = (*iter).second;
		// cout << "Vision Object: " << vo  << endl; // aldur 15.4.2006
        	// find  to the 'pol' entry in the object's section
        	Predicate::Iterator polIter = paramIter;
        	if (! predicate.FindParameter(polIter,"pol"))
                {
                    continue;
                }

        	// read the position vector
    	        VisionSense sense;
        	if (
            	    (! predicate.AdvanceValue(polIter,sense.distance)) ||
            	    (! predicate.AdvanceValue(polIter,sense.theta)) ||
            	    (! predicate.AdvanceValue(polIter,sense.phi))
            	    )
        	{
            	    continue;
        	}
	    
        	// update the vision map
                mVisionMap[vo] = sense;
	    }	
        }
}

void
Parser::ParseVision(const Predicate& predicate)
{
    ParseObjectVision(predicate);

    //
    // get our position. The current implementation of the worldmodel
    // relies on the VisionPerceptor that deliverd our position. In
    // future releases this should be replaced by a self localization
    // algorithm using the relative positions of some flags.
    //

    // find the PerfectVision data about the object
    Predicate::Iterator iter(predicate);

    // advance to the section about object 'name'
    if (! predicate.FindParameter(iter,"mypos"))
        {
            return;
        }

    // read my position
    VisionSense sense;

    predicate.GetValue(iter,mMyPos);
}

void Parser::Parse(const string& message)
{
    if(!usable) return;
    
    shared_ptr<PredicateList> predList =
        mParser->Parse(message);

    mPrimaryISModule->players.clear();
    
    
    if (predList.get() == 0)
    {
            return;
    }

    PredicateList& list = *predList;

    for (
         PredicateList::TList::const_iterator iter = list.begin();
         iter != list.end();
         ++iter
         )
        {
            const Predicate& predicate = (*iter);

            if (predicate.name == "GameState")
                {
                    ParseGameState(predicate);
                    continue;
                }

            // check for the Vision perceptor
            if (predicate.name == "Vision")
                {
                    ParseVision(predicate);
                    continue;
                }
        }

    // update our position
    CalcPlayerPosition();
}

Parser::VisionSense Parser::GetVisionSense(VisionObject obj)
{
    TVisionMap::iterator iter = mVisionMap.find(obj);

    if (iter == mVisionMap.end())
        {
	    return VisionSense();
        }

    return (*iter).second;
}

Vector3f Parser::GetPosition(VisionSense sense)
{    
    return mMyPos + GetDriveVec(sense) * sense.distance;  
    
}

Vector3f Parser::GetObjectPosition(VisionObject obj)
{
    return GetPosition(GetVisionSense(obj));
}


TPlayMode Parser::GetPlayMode()
{
    return mPrimaryISModule->mPlayMode;
}

Vector3f Parser::GetDriveVec(const Parser::VisionSense& vision)
{
    return Vector3f
        (
         vision.distance * gCos(gDegToRad(vision.theta)) *
         gSin(gDegToRad(90.0 - vision.phi)),

         vision.distance * gSin(gDegToRad(vision.theta)) *
         gSin(gDegToRad(90.0 - vision.phi)),

         vision.distance * gCos(gDegToRad(90.0 - vision.phi))
         );
}

void Parser::CalcPlayerPosition()
{
}

salt::Vector3f Parser::GetMyPosition()
{
    return mMyPos;
}

TTeamIndex Parser::GetMyTeam()
{
    return mPrimaryISModule->mTeamInfo.mTeamIndex;
}

float Parser::GetMinimalKickDistance()
{
    return mPrimaryISModule->mAgent.mAgentRadius + mPrimaryISModule->mBall.mBallRadius + 0.04;
}

int Parser::GetTeamUnum()
{
    return mPrimaryISModule->mTeamInfo.mTeamUnum;
}


void Parser::SetTeamName(std::string name){
    mPrimaryISModule->mTeamInfo.mTeamName =   name;
}

string Parser::GetTeamName(){
    return mPrimaryISModule->mTeamInfo.mTeamName;
}

void Parser:: SetOppTeamName(std::string name)
{
    mPrimaryISModule->mTeamInfo.mOppTeamName = name;
}

string Parser::GetOppTeamName()
{
       return mPrimaryISModule->mTeamInfo.mOppTeamName;    
}

// inicializacia Commserveru
void Parser::OnLink()
{
    mPrimaryISModule = shared_dynamic_cast<PrimaryISModule> (GetCore()->Get("/Agent/Sys/PrimaryISModule"));
    if (mPrimaryISModule.get() == 0)
        {
            // ak sa dostane sem, vznikla chyba pri inicializacii
	    printf("Parser::OnLink() : neni primaryIS\n");
        }
}

void Parser::OnUnlink()
{
    mPrimaryISModule.reset();
}
