/*

Trieda Parser (mnou vytvorena) vychadza z triedy WorldModel v agentteste
zmena spociva najma v tom, ze som odstranil vsetky odkazy na LogServer

momentalne je Parser na tom tak, ze zaroven obsahuje aj PrimaryISModule
ich oddelenie by nemalo byt nic technicky narocne, ale ta previazanost je tam silna, 
a kedze to doma nemam ako kompilovat, tak by som tam urcite urobil dost chyb

len mam pocit, ze chyba jedna zasadna vec - zistovanie polohy ostatnych hracov, ktorych hrac vidi
myslim ze v agenttest to nepotreboval, lebo kicknrun nepotreboval vediet, kde su ostatni hraci

*/

#ifndef PARSER_H
#define PARSER_H

#include <string>
#include <map>
#include <zeitgeist/leaf.h>
#include <soccer/soccertypes.h>
#include <oxygen/gamecontrolserver/predicate.h>
#include <oxygen/gamecontrolserver/baseparser.h>
#include "base_classes/ISModules/PrimaryISModule.h"

class Parser : public zeitgeist::Leaf
{
public:
    struct VisionSense
    {
        float distance;
        float theta;
        float phi;

        VisionSense() : distance(0), theta(0), phi(0) {};
    };
    
    struct PlayerVisionSense
    {
	string team;
	int id;
	float distance;
	float theta;
	float phi;
	PlayerVisionSense() : team(""), id(0), distance(0), theta(0), phi(0) {};
    };

    enum VisionObject
    {
        VO_BALL   = 0,
        VO_FLAG1L = 1,
        VO_FLAG1R = 2,
        VO_FLAG2L = 3,
        VO_FLAG2R = 4,
        VO_GOAL1L = 5,
        VO_GOAL1R = 6,
        VO_GOAL2L = 7,
        VO_GOAL2R = 8,
    };

public:
    Parser();
    virtual ~Parser();
    
    void OnLink();
    void OnUnlink();

    void Parse(const std::string& message);

    VisionSense GetVisionSense(VisionObject obj);

    // 4.6.2006 Matell
    //PrimaryISModule * mPrimaryISModule;
    shared_ptr<PrimaryISModule> mPrimaryISModule;
    
    salt::Vector3f GetPosition(VisionSense sense);
    salt::Vector3f GetObjectPosition(VisionObject obj);
    salt::Vector3f GetMyPosition();

    TPlayMode GetPlayMode();

    salt::Vector3f GetDriveVec(const Parser::VisionSense& vision);

    TTeamIndex GetMyTeam();

    float GetFieldWidth() { return mPrimaryISModule->mField.mFieldWidth; }
    float GetFieldLength() { return mPrimaryISModule->mField.mFieldLength; }

    float GetAgentRadius() { return mPrimaryISModule->mAgent.mAgentRadius; }
    float GetBallRadius() { return mPrimaryISModule->mBall.mBallRadius; }
    
    void ParseBallPosition();	// pv, 11-04-06
    void ParseL1Position();

    float GetMinimalKickDistance();
    int GetTeamUnum();
    
    void SetTeamName(std::string name);  
    std::string GetTeamName();             
    std::string GetOppTeamName();
    

protected:
    virtual bool ConstructInternal();

    void CalcPlayerPosition();

    void ParseObjectVision(const oxygen::Predicate& predicate);
    void ParseGameState(const oxygen::Predicate& predicate);
    void ParseVision(const oxygen::Predicate& predicate);

    void ParsePlayMode(const oxygen::Predicate& predicate);
    void ParseTeamIndex(const oxygen::Predicate& predicate);

    void SetupGameStateMap();
    void SetupVisionObjectMap();

protected:
    bool usable;
    // mapping from gamestate string to TPlayMode
    typedef std::map<std::string, TPlayMode> TPlayModeMap;
    TPlayModeMap mPlayModeMap;

    // mapping from vision object to VisionSense
    typedef std::map<VisionObject, VisionSense> TVisionMap;
    TVisionMap mVisionMap;

    // mapping from <ObjectName>+<ID> to VisionObject
    typedef std::map<std::string, VisionObject> TVisionObjectMap;
    TVisionObjectMap mVisionObjectMap;
    
    typedef std::map<std::string, PlayerVisionSense> TPlayerVisionObjectMap;
    TPlayerVisionObjectMap mPlayerVisionObjectMap;

    // parser instance
    boost::shared_ptr<oxygen::BaseParser> mParser;
    
    void SetOppTeamName(std::string name);

    // game state information
    float mTime;
    //TPlayMode mPlayMode;

    // the players position
    salt::Vector3f mMyPos;
};

DECLARE_CLASS(Parser);

#endif // PARSER_H
